/* tile definition file. This file is linked into the "tiletrans" utility,
 * which separates the tiles and generates the vultures_gametiles.{c,h} code
 *
 * Copyright (c) 2005 Daniel Thaler
 * This file may be freely distributed under the Nethack license*/

#include "tiletrans.h"
#include "config.h"
#include "monsym.h"
#include "objclass.h"

#undef EXPLOSION /* suppress a warning; we don't care about the original definition ...  */


#define MON_TINY_DX 57
#define MON_TINY_DY 62
#define MON_DX 116
#define MON_DY 126
#define MON_LARGE_DX 154
#define MON_LARGE_DY 165
#define MON_HUGE_DX 232
#define MON_HUGE_DY 252
#define FIG_DX 58
#define FIG_DY 63
#define FIG_LARGE_DX 77
#define FIG_LARGE_DY 84
#define FIG_HUGE_DX 116
#define FIG_HUGE_DY 126
#define CMAP_DX 116
#define CMAP_DY 126
#define OBJ_DX 96
#define OBJ_DY 102


const struct floorstyles fstyles[] = {
/* basename           x  y */
{"FLOOR_COBBLESTONE", 3, 3},
{"FLOOR_ROUGH",       3, 3},
{"FLOOR_CERAMIC",     3, 3},
{"FLOOR_LAVA",        3, 3},
{"FLOOR_WATER",       3, 3},
{"FLOOR_ICE",         3, 3},
{"FLOOR_MURAL",       3, 2},
{"FLOOR_MURAL2",      3, 2},
{"FLOOR_CARPET",      3, 2},
{"FLOOR_MOSS_COVERED",3, 3},
{"FLOOR_MARBLE",      3, 3},
{"FLOOR_ROUGH_LIT",   3, 3},
{"FLOOR_AIR",         3, 3},
{"FLOOR_DARK",        1, 1}
};


const char * floor_edges[] = {
	"FLOOR_EDGE_COBBLESTONE"
};



const char * wall_styles[] = {
	"WALL_BRICK",
	"WALL_BRICK_BANNER",
	"WALL_BRICK_PAINTING",
	"WALL_BRICK_POCKET",
	"WALL_BRICK_PILLAR",
	"WALL_MARBLE",
	"WALL_VINE_COVERED",
	"WALL_STUCCO",
	"WALL_ROUGH",
	"WALL_DARK",
	"WALL_LIGHT"
};


/* classdefmaps assign a default tile to an entire class of monsters or objects */
const struct classdefmap objclassmap[] = {
{ILLOBJ_CLASS,  "STRANGE_OBJECT"},
{WEAPON_CLASS,  "SPEAR"},
{ARMOR_CLASS,   "HELMET"},
{RING_CLASS,    "RIN_ADORNMENT"},
{AMULET_CLASS,  "AMULET_OF_ESP"},
{TOOL_CLASS,    "STRANGE_OBJECT"},
{FOOD_CLASS,    "FOOD_RATION"},
{POTION_CLASS,  "POT_WATER"},
{SCROLL_CLASS,  "SCR_BLANK_PAPER"},
{SPBOOK_CLASS,  "SPE_BLANK_PAPER"},
{WAND_CLASS,    "WAN_LIGHT"},
{COIN_CLASS,    "GOLD_PIECE"},
{GEM_CLASS,     "DIAMOND"},
{ROCK_CLASS,    "BOULDER"},
{BALL_CLASS,    "STRANGE_OBJECT"},
{CHAIN_CLASS,   "STRANGE_OBJECT"},
{VENOM_CLASS,   "POT_WATER"}
};

const struct classdefmap monclassmap[] = {
{S_ANT,           "PM_SOLDIER_ANT"},
{S_BLOB,          "PM_GREEN_SLIME"},
{S_COCKATRICE,    "PM_FLOATING_EYE"},
{S_DOG,           "PM_WOLF"},
{S_EYE,           "PM_FLOATING_EYE"},
{S_FELINE,        "PM_LARGE_CAT"},
{S_GREMLIN,       "PM_GNOME"},
{S_HUMANOID,      "PM_GOBLIN"},
{S_IMP,           "PM_GOBLIN"},
{S_JELLY,         "PM_GREEN_SLIME"},
{S_KOBOLD,        "PM_GOBLIN"},
{S_LEPRECHAUN,    "PM_GNOME"},
{S_MIMIC,         "PM_LARGE_MIMIC"},
{S_NYMPH,         "PM_WOOD_NYMPH"},
{S_ORC,           "PM_ORC"},
{S_QUADRUPED,     "PM_HORSE"},
{S_RODENT,        "PM_GIANT_RAT"},
{S_SPIDER,        "PM_GIANT_SPIDER"},
{S_UNICORN,       "PM_HORSE"},
{S_ANGEL,         "PM_ANGEL"},
{S_BAT,           "PM_VAMPIRE_BAT"},
{S_CENTAUR,       "PM_FOREST_CENTAUR"},
{S_DRAGON,        "PM_BLACK_DRAGON"},
{S_FUNGUS,        "PM_LICHEN"},
{S_GNOME,         "PM_GNOME"},
{S_GIANT,         "PM_GIANT"},
{S_JABBERWOCK,    "PM_FLOATING_EYE"},
{S_KOP,           "PM_KNIGHT"},
{S_LICH,          "PM_LICH"},
{S_MUMMY,         "PM_HUMAN_MUMMY"},
{S_NAGA,          "PM_GUARDIAN_NAGA"},
{S_OGRE,          "PM_OGRE"},
{S_PUDDING,       "PM_GREEN_SLIME"},
{S_SNAKE,         "PM_WATER_MOCCASIN"},
{S_TROLL,         "PM_TROLL"},
{S_WRAITH,        "PM_WRAITH"},
{S_ZOMBIE,        "PM_HUMAN_ZOMBIE"},
{S_GHOST,         "PM_GHOST"},
{S_GOLEM,         "PM_STONE_GOLEM"},
{S_DEMON,         "PM_WATER_DEMON"},
{S_LIZARD,        "PM_LIZARD"},
{S_VAMPIRE,       "PM_VAMPIRE"},
{S_WORM,          "PM_BABY_PURPLE_WORM"}
};



const struct eqmap tilemap[] = {
/*type       x      looks like     y*/

/* objects */
{T_OBJECT, "ELVEN_LEATHER_HELM",         "HELMET"},
{T_OBJECT, "ORCISH_HELM",                "HELMET"},
{T_OBJECT, "DWARVISH_IRON_HELM",         "HELMET"},
{T_OBJECT, "DENTED_POT",                 "HELMET"},
{T_OBJECT, "HELM_OF_BRILLIANCE",         "HELMET"},
{T_OBJECT, "HELM_OF_OPPOSITE_ALIGNMENT", "HELMET"},
{T_OBJECT, "HELM_OF_TELEPATHY",          "HELMET"},

{T_OBJECT, "SHORT_SWORD",          "LONG_SWORD"},
{T_OBJECT, "ELVEN_SHORT_SWORD",    "LONG_SWORD"},
{T_OBJECT, "ORCISH_SHORT_SWORD",   "LONG_SWORD"},
{T_OBJECT, "DWARVISH_SHORT_SWORD", "LONG_SWORD"},
{T_OBJECT, "SCIMITAR",             "LONG_SWORD"},
{T_OBJECT, "SILVER_SABER",         "LONG_SWORD"},
{T_OBJECT, "BROADSWORD",           "LONG_SWORD"},
{T_OBJECT, "ELVEN_BROADSWORD",     "LONG_SWORD"},
{T_OBJECT, "TWO_HANDED_SWORD",     "LONG_SWORD"},
{T_OBJECT, "TSURUGI",              "LONG_SWORD"},
{T_OBJECT, "RUNESWORD",            "LONG_SWORD"},

{T_OBJECT, "ELVEN_SPEAR",    "SPEAR"},
{T_OBJECT, "ORCISH_SPEAR",   "SPEAR"},
{T_OBJECT, "DWARVISH_SPEAR", "SPEAR"},
{T_OBJECT, "SILVER_SPEAR",   "SPEAR"},
{T_OBJECT, "JAVELIN",        "SPEAR"},
{T_OBJECT, "TRIDENT",        "SPEAR"},

{T_OBJECT, "ELVEN_DAGGER",  "DAGGER"},
{T_OBJECT, "ORCISH_DAGGER", "DAGGER"},
{T_OBJECT, "SILVER_DAGGER", "DAGGER"},
{T_OBJECT, "ATHAME",        "DAGGER"},
{T_OBJECT, "SCALPEL",       "DAGGER"},
{T_OBJECT, "KNIFE",         "DAGGER"},
{T_OBJECT, "STILETTO",      "DAGGER"},
{T_OBJECT, "WORM_TOOTH",    "DAGGER"},
{T_OBJECT, "CRYSKNIFE",     "DAGGER"},

{T_OBJECT, "BATTLE_AXE", "AXE"},

{T_OBJECT, "LUCERN_HAMMER", "WAR_HAMMER"},

{T_OBJECT, "IRON_SHOES",          "LOW_BOOTS"},
{T_OBJECT, "HIGH_BOOTS",          "LOW_BOOTS"},
{T_OBJECT, "SPEED_BOOTS",         "LOW_BOOTS"},
{T_OBJECT, "WATER_WALKING_BOOTS", "LOW_BOOTS"},
{T_OBJECT, "JUMPING_BOOTS",       "LOW_BOOTS"},
{T_OBJECT, "ELVEN_BOOTS",         "LOW_BOOTS"},
{T_OBJECT, "KICKING_BOOTS",       "LOW_BOOTS"},
{T_OBJECT, "FUMBLE_BOOTS",        "LOW_BOOTS"},
{T_OBJECT, "LEVITATION_BOOTS",    "LOW_BOOTS"},

{T_OBJECT, "MORNING_STAR", "MACE"},
{T_OBJECT, "FLAIL",        "MACE"},

{T_OBJECT, "AKLYS", "CLUB"},

{T_OBJECT, "STUDDED_LEATHER_ARMOR", "LEATHER_ARMOR"},
{T_OBJECT, "LEATHER_JACKET",        "LEATHER_ARMOR"},

{T_OBJECT, "GRAY_DRAGON_SCALE_MAIL",   "SCALE_MAIL"},
{T_OBJECT, "SILVER_DRAGON_SCALE_MAIL", "SCALE_MAIL"},
{T_OBJECT, "RED_DRAGON_SCALE_MAIL",    "SCALE_MAIL"},
{T_OBJECT, "WHITE_DRAGON_SCALE_MAIL",  "SCALE_MAIL"},
{T_OBJECT, "ORANGE_DRAGON_SCALE_MAIL", "SCALE_MAIL"},
{T_OBJECT, "BLACK_DRAGON_SCALE_MAIL",  "SCALE_MAIL"},
{T_OBJECT, "BLUE_DRAGON_SCALE_MAIL",   "SCALE_MAIL"},
{T_OBJECT, "GREEN_DRAGON_SCALE_MAIL",  "SCALE_MAIL"},
{T_OBJECT, "YELLOW_DRAGON_SCALE_MAIL", "SCALE_MAIL"},
/* next 2 are slashem only */
{T_OBJECT, "SHIMMERING_DRAGON_SCALE_MAIL","SCALE_MAIL"},
{T_OBJECT, "DEEP_DRAGON_SCALE_MAIL",      "SCALE_MAIL"},

{T_OBJECT, "SPLINT_MAIL",        "PLATE_MAIL"},
{T_OBJECT, "BANDED_MAIL",        "PLATE_MAIL"},

{T_OBJECT, "ORCISH_CHAIN_MAIL",       "CHAIN_MAIL"},
{T_OBJECT, "ELVEN_MITHRIL_COAT",      "CHAIN_MAIL"},
{T_OBJECT, "DARK_ELVEN_MITHRIL_COAT", "CHAIN_MAIL"},
{T_OBJECT, "DWARVISH_MITHRIL_COAT",   "CHAIN_MAIL"},

{T_OBJECT, "ORCISH_RING_MAIL", "RING_MAIL"},

{T_OBJECT, "GAUNTLETS_OF_FUMBLING",  "LEATHER_GLOVES"},
{T_OBJECT, "GAUNTLETS_OF_POWER",     "LEATHER_GLOVES"},
{T_OBJECT, "GAUNTLETS_OF_SWIMMING",  "LEATHER_GLOVES"},
{T_OBJECT, "GAUNTLETS_OF_DEXTERITY", "LEATHER_GLOVES"},
{T_OBJECT, "GAUNTLETS_OF_POWER",     "LEATHER_GLOVES"},

{T_OBJECT, "TALLOW_CANDLE", "WAX_CANDLE"},
{T_OBJECT, "MAGIC_CANDLE",  "WAX_CANDLE"}, /* slashem only */

{T_OBJECT, "ELVEN_BOW",  "BOW"},
{T_OBJECT, "ORCISH_BOW", "BOW"},
{T_OBJECT, "YUMI",       "BOW"},

{T_OBJECT, "MUMMY_WRAPPING",            "ELVEN_CLOAK"},
{T_OBJECT, "ORCISH_CLOAK",              "ELVEN_CLOAK"},
{T_OBJECT, "DWARVISH_CLOAK",            "ELVEN_CLOAK"},
{T_OBJECT, "OILSKIN_CLOAK",             "ELVEN_CLOAK"},
{T_OBJECT, "ROBE",                      "ELVEN_CLOAK"},
{T_OBJECT, "ALCHEMY_SMOCK",             "ELVEN_CLOAK"},
{T_OBJECT, "LEATHER_CLOAK",             "ELVEN_CLOAK"},
{T_OBJECT, "CLOAK_OF_PROTECTION",       "ELVEN_CLOAK"},
{T_OBJECT, "CLOAK_OF_INVISIBILITY",     "ELVEN_CLOAK"},
{T_OBJECT, "CLOAK_OF_MAGIC_RESISTANCE", "ELVEN_CLOAK"},
{T_OBJECT, "CLOAK_OF_DISPLACEMENT",     "ELVEN_CLOAK"},
{T_OBJECT, "POISONOUS_CLOAK",           "ELVEN_CLOAK"},
/* next 4 are slashem only */
{T_OBJECT, "ROBE_OF_PROTECTION",        "ELVEN_CLOAK"},
{T_OBJECT, "ROBE_OF_POWER",             "ELVEN_CLOAK"},
{T_OBJECT, "ROBE_OF_WEAKNESS",          "ELVEN_CLOAK"},
{T_OBJECT, "LAB_COAT",                  "ELVEN_CLOAK"},

{T_OBJECT, "DUNCE_CAP", "CORNUTHAUM"},

{T_OBJECT, "LOCK_PICK",  "SKELETON_KEY"},

{T_OBJECT, "DWARVISH_MATTOCK", "PICK_AXE"},

{T_OBJECT, "MAGIC_FLUTE", "WOODEN_FLUTE"},
{T_OBJECT, "MAGIC_HARP", "WOODEN_HARP"},

{T_OBJECT, "DRUM_OF_EARTHQUAKE", "LEATHER_DRUM"},

{T_OBJECT, "HORN_OF_PLENTY", "FROST_HORN"},
};



#define PLACEHOLDER(name)                {T_SPECIAL, name, NULL,     0,             0,             0,         0, __LINE__}
#define CMAP(name, filename, x, y)       {T_SPECIAL, name, filename, 1+(x*CMAP_DX), 1+(y*CMAP_DY), CMAP_DX-4, CMAP_DY-4, __LINE__}
#define SPECIALMON(name, filename, x, y) {T_SPECIAL, name, filename, 1+(x*MON_DX),  1+(y*MON_DY),  MON_DX-4,  MON_DY-4, __LINE__}
#define SPECIALOBJ(name, filename, x, y) {T_SPECIAL, name, filename, 1+(x*OBJ_DX),  1+(y*OBJ_DY),  OBJ_DX-4,  OBJ_DY-4, __LINE__}
#define ENGULF(name, filename, x, y)     {T_SPECIAL, name, filename, 1+(x*218),     1+(y*234),     214,       230, __LINE__}

#define EXPLOSION(name, filename, x, y)  {T_EXPL,    name, filename, 1+(x*CMAP_DX), 1+(y*CMAP_DY), CMAP_DX-4, CMAP_DY-4, __LINE__}

#define FLOOR_EDGE(name, filename, x, y) {T_DUNGEON, name, filename, 1+(x*116),     1+(y*53),      112,       49, __LINE__}
#define FLOOR(name, filename, x, y)      {T_FLOOR,   name, filename, 1+(x*116),     1+(y*53),      112,       49, __LINE__}
#define WALL(name, filename, x, y)       {T_WALL,    name, filename, 1+(x*60),      1+(y*126),     56,        122, __LINE__}

#define OBJECT(name, filename, x, y)     {T_OBJECT,  name, filename, 1+(x*OBJ_DX),  1+(y*OBJ_DY),  OBJ_DX-4,  OBJ_DY-4, __LINE__}

#define STATUE(name, filename, x, y, width, height) \
    {T_STATUE,  name, filename, 1+(x*width), 1+(y*height), width-4, height-4, __LINE__}

#define MONSTER(name, filename, x, y, width, height) \
    {T_MONSTER,  name, filename, 1+(x*width), 1+(y*height), width-4, height-4, __LINE__}

#define FIGURINE(name, filename, x, y, width, height) \
    {T_FIGURINE,  name, filename, 1+(x*width), 1+(y*height), width-4, height-4, __LINE__}

#define CURSOR(name, x, y, w, h) {T_CURSOR, "CURSOR_"name, "cursors.png", x, y, w, h, __LINE__}


const struct tiledef tiledefinitions[] = {
/* Special tiles: anything that does not directly correspond to a monster or object
 * NAME may be anything; it will return in the output file as VTILE_<NAME> */
/* special: NAME                   FILENAME          top left x coord      top left y coord    width       height*/
PLACEHOLDER("NONE"),
PLACEHOLDER("PLAYER_INVIS"),
PLACEHOLDER("WALL_GENERIC"),
/* floor placeholders: these are used until we determine the actual tile in the floor pattern */
PLACEHOLDER("FLOOR_COBBLESTONE"),
PLACEHOLDER("FLOOR_ROUGH"),
PLACEHOLDER("FLOOR_WATER"),
PLACEHOLDER("FLOOR_ICE"),
PLACEHOLDER("FLOOR_LAVA"),
PLACEHOLDER("FLOOR_ROUGH_LIT"),
PLACEHOLDER("FLOOR_AIR"),
PLACEHOLDER("FLOOR_DARK"),
CMAP("FLOOR_NOT_VISIBLE",    "misc_07.png", 3, 3),
CMAP("DOOR_WOOD_BROKEN",     "misc_07.png", 0, 0),
CMAP("HDOOR_WOOD_CLOSED",    "misc_07.png", 1, 0),
CMAP("VDOOR_WOOD_CLOSED",    "misc_07.png", 2, 0),
CMAP("VDOOR_WOOD_OPEN",      "misc_07.png", 3, 0),
CMAP("HDOOR_WOOD_OPEN",      "misc_07.png", 4, 0),
CMAP("TRAP_BEAR",            "misc_07.png", 5, 0),
CMAP("GRAVE",                "misc_07.png", 0, 1),
CMAP("ALTAR",                "misc_07.png", 1, 1),
CMAP("FOUNTAIN",             "misc_07.png", 2, 1),
CMAP("STAIRS_UP",            "misc_07.png", 3, 1),
CMAP("STAIRS_DOWN",          "misc_07.png", 4, 1),
CMAP("SINK",                 "misc_07.png", 5, 1),
CMAP("GAS_TRAP",             "misc_07.png", 0, 2),
CMAP("TRAP_PIT",             "misc_07.png", 1, 2),
CMAP("TRAP_POLYMORPH",       "misc_07.png", 2, 2),
CMAP("TREE",                 "misc_07.png", 3, 2),
CMAP("TRAP_MAGIC",           "misc_07.png", 5, 2),
CMAP("TRAP_DOOR",            "misc_07.png", 0, 3),
CMAP("TRAP_WATER",           "misc_07.png", 1, 3),
CMAP("TRAP_TELEPORTER",      "misc_07.png", 2, 3),
CMAP("UNMAPPED_AREA",        "misc_07.png", 4, 3),
CMAP("HILITE_PET",           "misc_07.png", 5, 3),
CMAP("BARS",                 "misc_c7.png", 0, 0),
CMAP("THRONE",               "misc_c7.png", 1, 0),
CMAP("HIGHLIGHT_BLESS",      "misc_c7.png", 2, 0),
CMAP("TRAP_ANTI_MAGIC",      "misc_c7.png", 0, 1),
CMAP("TRAP_ARROW",           "misc_c7.png", 1, 1),
CMAP("HIGHLIGHT_CURSE",      "misc_c7.png", 2, 1),
CMAP("TRAP_FIRE",            "misc_c7.png", 0, 2),
CMAP("ROLLING_BOULDER_TRAP", "misc_c7.png", 1, 2),
CMAP("TRAP_SLEEPGAS",        "misc_c7.png", 2, 2),
CMAP("ZAP_SLANT_RIGHT",      "misc_c7.png", 0, 3),
CMAP("ZAP_SLANT_LEFT",       "misc_c7.png", 1, 3),
CMAP("ZAP_HORIZONTAL",       "misc_c7.png", 2, 3),
CMAP("ZAP_VERTICAL",         "misc_c7.png", 3, 3),
CMAP("LADDER_UP",            "misc_c7.png", 4, 3),
CMAP("LADDER_DOWN",          "misc_c7.png", 5, 3),
CMAP("RESIST_SPELL_1",       "misc_c1.png", 0, 0),
CMAP("RESIST_SPELL_2",       "misc_c1.png", 1, 0),
CMAP("RESIST_SPELL_3",       "misc_c1.png", 2, 0),
CMAP("RESIST_SPELL_4",       "misc_c1.png", 3, 0),
CMAP("WEB_TRAP",             "misc_c1.png", 4, 0),
CMAP("DART_TRAP",            "misc_c1.png", 5, 0),
CMAP("FALLING_ROCK_TRAP",    "misc_c1.png", 0, 1),
CMAP("SQUEAKY_BOARD",        "misc_c1.png", 1, 1),
CMAP("MAGIC_PORTAL",         "misc_c1.png", 3, 1),
CMAP("SPIKED_PIT",           "misc_c1.png", 4, 1),
CMAP("HOLE",                 "misc_c1.png", 5, 1),
CMAP("LEVEL_TELEPORTER",     "misc_c1.png", 0, 2),
CMAP("MAGIC_TRAP",           "misc_c1.png", 1, 2),
CMAP("DIGBEAM",              "misc_c1.png", 2, 2),
CMAP("FLASHBEAM",            "misc_c1.png", 3, 2),
CMAP("BOOMLEFT",             "misc_c1.png", 4, 2),
CMAP("BOOMRIGHT",            "misc_c1.png", 5, 2),
CMAP("HCDBRIDGE",            "misc_c1.png", 0, 3),
CMAP("VCDBRIDGE",            "misc_c1.png", 1, 3),
CMAP("VODBRIDGE",            "misc_c1.png", 2, 3),
CMAP("HODBRIDGE",            "misc_c1.png", 3, 3),
CMAP("CLOUD",                "misc_c1.png", 4, 3),
CMAP("OFF_MAP",              "misc_c1.png", 5, 3),
SPECIALOBJ("STONE",                "object2.png", 3, 4),
SPECIALOBJ("UNIDENTIFIED_BAG",     "object4.png", 2, 4),
SPECIALOBJ("LAND_MINE",            "object5.png", 7, 1),
SPECIALMON("LAWFUL_PRIEST",        "monster1.png", 5, 2),
SPECIALMON("CHAOTIC_PRIEST",       "monster1.png", 0, 3),
SPECIALMON("NEUTRAL_PRIEST",       "monster1.png", 1, 3),
SPECIALMON("UNALIGNED_PRIEST",     "monster1.png", 2, 3),
SPECIALMON("ROGUE_LEVEL_A",        "monstermisc1.png", 0, 0),
SPECIALMON("ROGUE_LEVEL_B",        "monstermisc1.png", 1, 0),
SPECIALMON("ROGUE_LEVEL_C",        "monstermisc1.png", 2, 0),
SPECIALMON("ROGUE_LEVEL_D",        "monstermisc1.png", 3, 0),
SPECIALMON("ROGUE_LEVEL_E",        "monstermisc1.png", 4, 0),
SPECIALMON("ROGUE_LEVEL_F",        "monstermisc1.png", 5, 0),
SPECIALMON("ROGUE_LEVEL_G",        "monstermisc1.png", 0, 1),
SPECIALMON("ROGUE_LEVEL_H",        "monstermisc1.png", 1, 1),
SPECIALMON("ROGUE_LEVEL_I",        "monstermisc1.png", 2, 1),
SPECIALMON("ROGUE_LEVEL_J",        "monstermisc1.png", 3, 1),
SPECIALMON("ROGUE_LEVEL_K",        "monstermisc1.png", 4, 1),
SPECIALMON("ROGUE_LEVEL_L",        "monstermisc1.png", 5, 1),
SPECIALMON("ROGUE_LEVEL_M",        "monstermisc1.png", 0, 2),
SPECIALMON("ROGUE_LEVEL_N",        "monstermisc1.png", 1, 2),
SPECIALMON("ROGUE_LEVEL_O",        "monstermisc1.png", 2, 2),
SPECIALMON("ROGUE_LEVEL_P",        "monstermisc1.png", 3, 2),
SPECIALMON("ROGUE_LEVEL_Q",        "monstermisc1.png", 4, 2),
SPECIALMON("ROGUE_LEVEL_R",        "monstermisc1.png", 5, 2),
SPECIALMON("ROGUE_LEVEL_S",        "monstermisc1.png", 0, 3),
SPECIALMON("ROGUE_LEVEL_T",        "monstermisc1.png", 1, 3),
SPECIALMON("ROGUE_LEVEL_U",        "monstermisc1.png", 2, 3),
SPECIALMON("ROGUE_LEVEL_V",        "monstermisc1.png", 3, 3),
SPECIALMON("ROGUE_LEVEL_W",        "monstermisc1.png", 4, 3),
SPECIALMON("ROGUE_LEVEL_X",        "monstermisc1.png", 5, 3),
SPECIALMON("ROGUE_LEVEL_Y",        "monstermisc2.png", 0, 0),
SPECIALMON("ROGUE_LEVEL_Z",        "monstermisc2.png", 1, 0),
ENGULF("ENGULF_FIRE_VORTEX",   "engulf.png", 0, 0),
ENGULF("ENGULF_FOG_CLOUD",     "engulf.png", 1, 0),
ENGULF("ENGULF_AIR_ELEMENTAL", "engulf.png", 2, 0),
ENGULF("ENGULF_STEAM_VORTEX",  "engulf.png", 0, 1),
ENGULF("ENGULF_PURPLE_WORM",   "engulf.png", 1, 1),
ENGULF("ENGULF_JUIBLEX",       "engulf.png", 2, 1),
ENGULF("ENGULF_OCHRE_JELLY",   "engulf2.png", 0, 0),
ENGULF("ENGULF_LURKER_ABOVE",  "engulf2.png", 1, 0),
ENGULF("ENGULF_TRAPPER",       "engulf2.png", 2, 0),
ENGULF("ENGULF_DUST_VORTEX",   "engulf2.png", 0, 1),
ENGULF("ENGULF_ICE_VORTEX",    "engulf2.png", 1, 1),
ENGULF("ENGULF_ENERGY_VORTEX", "engulf2.png", 2, 1),
CMAP("WARNLEV_1",            "unknown.png", 5, 0),
CMAP("WARNLEV_2",            "unknown.png", 4, 0),
CMAP("WARNLEV_3",            "unknown.png", 3, 0),
CMAP("WARNLEV_4",            "unknown.png", 2, 0),
CMAP("WARNLEV_5",            "unknown.png", 1, 0),
CMAP("WARNLEV_6",            "unknown.png", 0, 0),
CMAP("INVISIBLE_MONSTER",    "unknown.png", 3, 2),
CMAP("STINKING_CLOUD",       "misc_c8.png", 0, 3),
CMAP("TOILET",               "misc_c8.png", 1, 3), /* slashem only */


/* Explostion tiles */
EXPLOSION("FIERY_1",          "misc_c7.png", 3, 0),
EXPLOSION("FIERY_2",          "misc_c7.png", 4, 0),
EXPLOSION("FIERY_3",          "misc_c7.png", 5, 0),
EXPLOSION("FIERY_4",          "misc_c7.png", 3, 1),
EXPLOSION("FIERY_5",          "misc_c7.png", 4, 1),
EXPLOSION("FIERY_6",          "misc_c7.png", 5, 1),
EXPLOSION("FIERY_7",          "misc_c7.png", 3, 2),
EXPLOSION("FIERY_8",          "misc_c7.png", 4, 2),
EXPLOSION("FIERY_9",          "misc_c7.png", 5, 2),
EXPLOSION("MAGICAL_1",        "misc_c8.png", 0, 0),
EXPLOSION("MAGICAL_2",        "misc_c8.png", 1, 0),
EXPLOSION("MAGICAL_3",        "misc_c8.png", 2, 0),
EXPLOSION("MAGICAL_4",        "misc_c8.png", 0, 1),
EXPLOSION("MAGICAL_5",        "misc_c8.png", 1, 1),
EXPLOSION("MAGICAL_6",        "misc_c8.png", 2, 1),
EXPLOSION("MAGICAL_7",        "misc_c8.png", 0, 2),
EXPLOSION("MAGICAL_8",        "misc_c8.png", 1, 2),
EXPLOSION("MAGICAL_9",        "misc_c8.png", 2, 2),
EXPLOSION("FROSTY_1",         "misc_c8.png", 3, 0),
EXPLOSION("FROSTY_2",         "misc_c8.png", 4, 0),
EXPLOSION("FROSTY_3",         "misc_c8.png", 5, 0),
EXPLOSION("FROSTY_4",         "misc_c8.png", 3, 1),
EXPLOSION("FROSTY_5",         "misc_c8.png", 4, 1),
EXPLOSION("FROSTY_6",         "misc_c8.png", 5, 1),
EXPLOSION("FROSTY_7",         "misc_c8.png", 3, 2),
EXPLOSION("FROSTY_8",         "misc_c8.png", 4, 2),
EXPLOSION("FROSTY_9",         "misc_c8.png", 5, 2),

/* Dungeon tiles: Walls, floors, floor edges
 * NAME is always a basename + extensions; it is used to find what group a tile belongs to but it is not output */
/* floor edges: name = basename + "_" + 1-12 */
FLOOR_EDGE("FLOOR_EDGE_COBBLESTONE_1", "misc_12.png", 0, 9),
FLOOR_EDGE("FLOOR_EDGE_COBBLESTONE_2", "misc_12.png", 1, 9),
FLOOR_EDGE("FLOOR_EDGE_COBBLESTONE_3", "misc_12.png", 1, 10),
FLOOR_EDGE("FLOOR_EDGE_COBBLESTONE_4", "misc_12.png", 0, 10),
FLOOR_EDGE("FLOOR_EDGE_COBBLESTONE_5", "misc_12.png", 2, 9),
FLOOR_EDGE("FLOOR_EDGE_COBBLESTONE_6", "misc_12.png", 3, 9),
FLOOR_EDGE("FLOOR_EDGE_COBBLESTONE_7", "misc_12.png", 3, 10),
FLOOR_EDGE("FLOOR_EDGE_COBBLESTONE_8", "misc_12.png", 2, 10),
FLOOR_EDGE("FLOOR_EDGE_COBBLESTONE_9", "misc_12.png", 4, 9),
FLOOR_EDGE("FLOOR_EDGE_COBBLESTONE_10","misc_12.png", 5, 9),
FLOOR_EDGE("FLOOR_EDGE_COBBLESTONE_11","misc_12.png", 5, 10),
FLOOR_EDGE("FLOOR_EDGE_COBBLESTONE_12","misc_12.png", 4, 10),
/* floors: name = patternname + "_" + x-coord + "_" + y-coord */
FLOOR("FLOOR_COBBLESTONE_0_0","misc_12.png", 0, 0),
FLOOR("FLOOR_COBBLESTONE_1_0","misc_12.png", 1, 0),
FLOOR("FLOOR_COBBLESTONE_2_0","misc_12.png", 2, 0),
FLOOR("FLOOR_COBBLESTONE_0_1","misc_12.png", 0, 1),
FLOOR("FLOOR_COBBLESTONE_1_1","misc_12.png", 1, 1),
FLOOR("FLOOR_COBBLESTONE_2_1","misc_12.png", 2, 1),
FLOOR("FLOOR_COBBLESTONE_0_2","misc_12.png", 0, 2),
FLOOR("FLOOR_COBBLESTONE_1_2","misc_12.png", 1, 2),
FLOOR("FLOOR_COBBLESTONE_2_2","misc_12.png", 2, 2),
FLOOR("FLOOR_ROUGH_0_0",      "misc_12.png", 0, 3),
FLOOR("FLOOR_ROUGH_1_0",      "misc_12.png", 1, 3),
FLOOR("FLOOR_ROUGH_2_0",      "misc_12.png", 2, 3),
FLOOR("FLOOR_ROUGH_0_1",      "misc_12.png", 0, 4),
FLOOR("FLOOR_ROUGH_1_1",      "misc_12.png", 1, 4),
FLOOR("FLOOR_ROUGH_2_1",      "misc_12.png", 2, 4),
FLOOR("FLOOR_ROUGH_0_2",      "misc_12.png", 0, 5),
FLOOR("FLOOR_ROUGH_1_2",      "misc_12.png", 1, 5),
FLOOR("FLOOR_ROUGH_2_2",      "misc_12.png", 2, 5),
FLOOR("FLOOR_CERAMIC_0_0",    "misc_12.png", 0, 6),
FLOOR("FLOOR_CERAMIC_1_0",    "misc_12.png", 1, 6),
FLOOR("FLOOR_CERAMIC_2_0",    "misc_12.png", 2, 6),
FLOOR("FLOOR_CERAMIC_0_1",    "misc_12.png", 0, 7),
FLOOR("FLOOR_CERAMIC_1_1",    "misc_12.png", 1, 7),
FLOOR("FLOOR_CERAMIC_2_1",    "misc_12.png", 2, 7),
FLOOR("FLOOR_CERAMIC_0_2",    "misc_12.png", 0, 8),
FLOOR("FLOOR_CERAMIC_1_2",    "misc_12.png", 1, 8),
FLOOR("FLOOR_CERAMIC_2_2",    "misc_12.png", 2, 8),
FLOOR("FLOOR_LAVA_0_0",       "misc_12.png", 3, 0),
FLOOR("FLOOR_LAVA_1_0",       "misc_12.png", 4, 0),
FLOOR("FLOOR_LAVA_2_0",       "misc_12.png", 5, 0),
FLOOR("FLOOR_LAVA_0_1",       "misc_12.png", 3, 1),
FLOOR("FLOOR_LAVA_1_1",       "misc_12.png", 4, 1),
FLOOR("FLOOR_LAVA_2_1",       "misc_12.png", 5, 1),
FLOOR("FLOOR_LAVA_0_2",       "misc_12.png", 3, 2),
FLOOR("FLOOR_LAVA_1_2",       "misc_12.png", 4, 2),
FLOOR("FLOOR_LAVA_2_2",       "misc_12.png", 5, 2),
FLOOR("FLOOR_WATER_0_0",      "misc_12.png", 3, 3),
FLOOR("FLOOR_WATER_1_0",      "misc_12.png", 4, 3),
FLOOR("FLOOR_WATER_2_0",      "misc_12.png", 5, 3),
FLOOR("FLOOR_WATER_0_1",      "misc_12.png", 3, 4),
FLOOR("FLOOR_WATER_1_1",      "misc_12.png", 4, 4),
FLOOR("FLOOR_WATER_2_1",      "misc_12.png", 5, 4),
FLOOR("FLOOR_WATER_0_2",      "misc_12.png", 3, 5),
FLOOR("FLOOR_WATER_1_2",      "misc_12.png", 4, 5),
FLOOR("FLOOR_WATER_2_2",      "misc_12.png", 5, 5),
FLOOR("FLOOR_ICE_0_0",        "misc_12.png", 3, 6),
FLOOR("FLOOR_ICE_1_0",        "misc_12.png", 4, 6),
FLOOR("FLOOR_ICE_2_0",        "misc_12.png", 5, 6),
FLOOR("FLOOR_ICE_0_1",        "misc_12.png", 3, 7),
FLOOR("FLOOR_ICE_1_1",        "misc_12.png", 4, 7),
FLOOR("FLOOR_ICE_2_1",        "misc_12.png", 5, 7),
FLOOR("FLOOR_ICE_0_2",        "misc_12.png", 3, 8),
FLOOR("FLOOR_ICE_1_2",        "misc_12.png", 4, 8),
FLOOR("FLOOR_ICE_2_2",        "misc_12.png", 5, 8),
FLOOR("FLOOR_MURAL_0_0",      "misc_13.png", 0, 0),
FLOOR("FLOOR_MURAL_1_0",      "misc_13.png", 1, 0),
FLOOR("FLOOR_MURAL_2_0",      "misc_13.png", 2, 0),
FLOOR("FLOOR_MURAL_0_1",      "misc_13.png", 0, 1),
FLOOR("FLOOR_MURAL_1_1",      "misc_13.png", 1, 1),
FLOOR("FLOOR_MURAL_2_1",      "misc_13.png", 2, 1),
FLOOR("FLOOR_MURAL2_0_0",     "misc_13.png", 3, 0),
FLOOR("FLOOR_MURAL2_1_0",     "misc_13.png", 4, 0),
FLOOR("FLOOR_MURAL2_2_0",     "misc_13.png", 5, 0),
FLOOR("FLOOR_MURAL2_0_1",     "misc_13.png", 3, 1),
FLOOR("FLOOR_MURAL2_1_1",     "misc_13.png", 4, 1),
FLOOR("FLOOR_MURAL2_2_1",     "misc_13.png", 5, 1),
FLOOR("FLOOR_CARPET_0_0",     "misc_13.png", 0, 2),
FLOOR("FLOOR_CARPET_1_0",     "misc_13.png", 1, 2),
FLOOR("FLOOR_CARPET_2_0",     "misc_13.png", 2, 2),
FLOOR("FLOOR_CARPET_0_1",     "misc_13.png", 0, 3),
FLOOR("FLOOR_CARPET_1_1",     "misc_13.png", 1, 3),
FLOOR("FLOOR_CARPET_2_1",     "misc_13.png", 2, 3),
FLOOR("FLOOR_MOSS_COVERED_0_0","misc_13.png", 0, 4),
FLOOR("FLOOR_MOSS_COVERED_1_0","misc_13.png", 1, 4),
FLOOR("FLOOR_MOSS_COVERED_2_0","misc_13.png", 2, 4),
FLOOR("FLOOR_MOSS_COVERED_0_1","misc_13.png", 0, 5),
FLOOR("FLOOR_MOSS_COVERED_1_1","misc_13.png", 1, 5),
FLOOR("FLOOR_MOSS_COVERED_2_1","misc_13.png", 2, 5),
FLOOR("FLOOR_MOSS_COVERED_0_2","misc_13.png", 0, 6),
FLOOR("FLOOR_MOSS_COVERED_1_2","misc_13.png", 1, 6),
FLOOR("FLOOR_MOSS_COVERED_2_2","misc_13.png", 2, 6),
FLOOR("FLOOR_MARBLE_0_0",     "misc_13.png", 3, 4),
FLOOR("FLOOR_MARBLE_1_0",     "misc_13.png", 4, 4),
FLOOR("FLOOR_MARBLE_2_0",     "misc_13.png", 5, 4),
FLOOR("FLOOR_MARBLE_0_1",     "misc_13.png", 3, 5),
FLOOR("FLOOR_MARBLE_1_1",     "misc_13.png", 4, 5),
FLOOR("FLOOR_MARBLE_2_1",     "misc_13.png", 5, 5),
FLOOR("FLOOR_MARBLE_0_2",     "misc_13.png", 3, 6),
FLOOR("FLOOR_MARBLE_1_2",     "misc_13.png", 4, 6),
FLOOR("FLOOR_MARBLE_2_2",     "misc_13.png", 5, 6),
FLOOR("FLOOR_ROUGH_LIT_0_0",  "misc_13.png", 0, 7),
FLOOR("FLOOR_ROUGH_LIT_1_0",  "misc_13.png", 1, 7),
FLOOR("FLOOR_ROUGH_LIT_2_0",  "misc_13.png", 2, 7),
FLOOR("FLOOR_ROUGH_LIT_0_1",  "misc_13.png", 0, 8),
FLOOR("FLOOR_ROUGH_LIT_1_1",  "misc_13.png", 1, 8),
FLOOR("FLOOR_ROUGH_LIT_2_1",  "misc_13.png", 2, 8),
FLOOR("FLOOR_ROUGH_LIT_0_2",  "misc_13.png", 0, 9),
FLOOR("FLOOR_ROUGH_LIT_1_2",  "misc_13.png", 1, 9),
FLOOR("FLOOR_ROUGH_LIT_2_2",  "misc_13.png", 2, 9),
FLOOR("FLOOR_AIR_0_0",        "misc_13.png", 3, 7),
FLOOR("FLOOR_AIR_1_0",        "misc_13.png", 4, 7),
FLOOR("FLOOR_AIR_2_0",        "misc_13.png", 5, 7),
FLOOR("FLOOR_AIR_0_1",        "misc_13.png", 3, 8),
FLOOR("FLOOR_AIR_1_1",        "misc_13.png", 4, 8),
FLOOR("FLOOR_AIR_2_1",        "misc_13.png", 5, 8),
FLOOR("FLOOR_AIR_0_2",        "misc_13.png", 3, 9),
FLOOR("FLOOR_AIR_1_2",        "misc_13.png", 4, 9),
FLOOR("FLOOR_AIR_2_2",        "misc_13.png", 5, 9),
FLOOR("FLOOR_DARK_0_0",       "misc_13.png", 0, 10),
/* walls: name = basename + "_" + (F|H|T) + "_" + (W|N|S|E) 
 * where (F|H) stands for full height/half height and (W|N|S|E) is west/north/south/east */
WALL("WALL_BRICK_F_W",         "walls02a.png", 0, 0),
WALL("WALL_BRICK_F_N",         "walls02a.png", 1, 0),
WALL("WALL_BRICK_F_S",         "walls02a.png", 2, 0),
WALL("WALL_BRICK_F_E",         "walls02a.png", 3, 0),
WALL("WALL_BRICK_BANNER_F_W",  "walls02a.png", 4, 0),
WALL("WALL_BRICK_BANNER_F_N",  "walls02a.png", 5, 0),
WALL("WALL_BRICK_BANNER_F_S",  "walls02a.png", 6, 0),
WALL("WALL_BRICK_BANNER_F_E",  "walls02a.png", 7, 0),
WALL("WALL_BRICK_POCKET_F_W",  "walls02a.png", 8, 0),
WALL("WALL_BRICK_POCKET_F_N",  "walls02a.png", 9, 0),
WALL("WALL_BRICK_POCKET_F_S",  "walls02a.png", 10, 0),
WALL("WALL_BRICK_POCKET_F_E",  "walls02a.png", 11, 0),
WALL("WALL_BRICK_PAINTING_F_W","walls02a.png", 0, 1),
WALL("WALL_BRICK_PAINTING_F_N","walls02a.png", 1, 1),
WALL("WALL_BRICK_PAINTING_F_S","walls02a.png", 2, 1),
WALL("WALL_BRICK_PAINTING_F_E","walls02a.png", 3, 1),
WALL("WALL_BRICK_PILLAR_F_W",  "walls02a.png", 4, 1),
WALL("WALL_BRICK_PILLAR_F_N",  "walls02a.png", 5, 1),
WALL("WALL_BRICK_PILLAR_F_S",  "walls02a.png", 6, 1),
WALL("WALL_BRICK_PILLAR_F_E",  "walls02a.png", 7, 1),
WALL("WALL_VINE_COVERED_F_W",  "walls02a.png", 8, 1),
WALL("WALL_VINE_COVERED_F_N",  "walls02a.png", 9, 1),
WALL("WALL_VINE_COVERED_F_S",  "walls02a.png", 10, 1),
WALL("WALL_VINE_COVERED_F_E",  "walls02a.png", 11, 1),
WALL("WALL_MARBLE_F_W",        "walls02a.png", 0, 2),
WALL("WALL_MARBLE_F_N",        "walls02a.png", 1, 2),
WALL("WALL_MARBLE_F_S",        "walls02a.png", 2, 2),
WALL("WALL_MARBLE_F_E",        "walls02a.png", 3, 2),
WALL("WALL_DARK_F_W",          "walls02a.png", 4, 2),
WALL("WALL_DARK_F_N",          "walls02a.png", 5, 2),
WALL("WALL_DARK_F_S",          "walls02a.png", 6, 2),
WALL("WALL_DARK_F_E",          "walls02a.png", 7, 2),
WALL("WALL_STUCCO_F_W",        "walls02a.png", 8, 2),
WALL("WALL_STUCCO_F_N",        "walls02a.png", 9, 2),
WALL("WALL_STUCCO_F_S",        "walls02a.png", 10, 2),
WALL("WALL_STUCCO_F_E",        "walls02a.png", 11, 2),
WALL("WALL_LIGHT_F_W",         "walls02a.png", 4, 3),
WALL("WALL_LIGHT_F_N",         "walls02a.png", 5, 3),
WALL("WALL_LIGHT_F_S",         "walls02a.png", 6, 3),
WALL("WALL_LIGHT_F_E",         "walls02a.png", 7, 3),
WALL("WALL_ROUGH_F_W",         "walls02a.png", 8, 3),
WALL("WALL_ROUGH_F_N",         "walls02a.png", 9, 3),
WALL("WALL_ROUGH_F_S",         "walls02a.png", 10, 3),
WALL("WALL_ROUGH_F_E",         "walls02a.png", 11, 3),
WALL("WALL_BRICK_H_W",         "walls02c.png", 0, 0),
WALL("WALL_BRICK_H_N",         "walls02c.png", 1, 0),
WALL("WALL_BRICK_H_S",         "walls02c.png", 2, 0),
WALL("WALL_BRICK_H_E",         "walls02c.png", 3, 0),
WALL("WALL_BRICK_BANNER_H_W",  "walls02c.png", 4, 0),
WALL("WALL_BRICK_BANNER_H_N",  "walls02c.png", 5, 0),
WALL("WALL_BRICK_BANNER_H_S",  "walls02c.png", 6, 0),
WALL("WALL_BRICK_BANNER_H_E",  "walls02c.png", 7, 0),
WALL("WALL_BRICK_POCKET_H_W",  "walls02c.png", 8, 0),
WALL("WALL_BRICK_POCKET_H_N",  "walls02c.png", 9, 0),
WALL("WALL_BRICK_POCKET_H_S",  "walls02c.png", 10, 0),
WALL("WALL_BRICK_POCKET_H_E",  "walls02c.png", 11, 0),
WALL("WALL_BRICK_PAINTING_H_W","walls02c.png", 0, 1),
WALL("WALL_BRICK_PAINTING_H_N","walls02c.png", 1, 1),
WALL("WALL_BRICK_PAINTING_H_S","walls02c.png", 2, 1),
WALL("WALL_BRICK_PAINTING_H_E","walls02c.png", 3, 1),
WALL("WALL_BRICK_PILLAR_H_W",  "walls02c.png", 4, 1),
WALL("WALL_BRICK_PILLAR_H_N",  "walls02c.png", 5, 1),
WALL("WALL_BRICK_PILLAR_H_S",  "walls02c.png", 6, 1),
WALL("WALL_BRICK_PILLAR_H_E",  "walls02c.png", 7, 1),
WALL("WALL_VINE_COVERED_H_W",  "walls02c.png", 8, 1),
WALL("WALL_VINE_COVERED_H_N",  "walls02c.png", 9, 1),
WALL("WALL_VINE_COVERED_H_S",  "walls02c.png", 10, 1),
WALL("WALL_VINE_COVERED_H_E",  "walls02c.png", 11, 1),
WALL("WALL_MARBLE_H_W",        "walls02c.png", 0, 2),
WALL("WALL_MARBLE_H_N",        "walls02c.png", 1, 2),
WALL("WALL_MARBLE_H_S",        "walls02c.png", 2, 2),
WALL("WALL_MARBLE_H_E",        "walls02c.png", 3, 2),
WALL("WALL_DARK_H_W",          "walls02c.png", 4, 2),
WALL("WALL_DARK_H_N",          "walls02c.png", 5, 2),
WALL("WALL_DARK_H_S",          "walls02c.png", 6, 2),
WALL("WALL_DARK_H_E",          "walls02c.png", 7, 2),
WALL("WALL_STUCCO_H_W",        "walls02c.png", 8, 2),
WALL("WALL_STUCCO_H_N",        "walls02c.png", 9, 2),
WALL("WALL_STUCCO_H_S",        "walls02c.png", 10, 2),
WALL("WALL_STUCCO_H_E",        "walls02c.png", 11, 2),
WALL("WALL_LIGHT_H_W",         "walls02c.png", 4, 3),
WALL("WALL_LIGHT_H_N",         "walls02c.png", 5, 3),
WALL("WALL_LIGHT_H_S",         "walls02c.png", 6, 3),
WALL("WALL_LIGHT_H_E",         "walls02c.png", 7, 3),
WALL("WALL_ROUGH_H_W",         "walls02c.png", 8, 3),
WALL("WALL_ROUGH_H_N",         "walls02c.png", 9, 3),
WALL("WALL_ROUGH_H_S",         "walls02c.png", 10, 3),
WALL("WALL_ROUGH_H_E",         "walls02c.png", 11, 3),



/* The name field is the name of a define in onames.h, _as a string_ !
 * It's okay for an object to be assigned to a tile that matches its description
 *(ie a potion of oil is assigned to a picture of a murky potion) because we shuffle those later */
{T_OBJECT, "LAND_MINE",            "misc_c1.png",   1+(2*CMAP_DX),       1+(1*CMAP_DY),      CMAP_DX-4,  CMAP_DY-4},
OBJECT("STRANGE_OBJECT",       "object1.png", 0, 0),
OBJECT("SKELETON_KEY",         "object1.png", 7, 0),
OBJECT("BOULDER",              "object1.png", 1, 0),
OBJECT("CORPSE",               "object1.png", 2, 0),
OBJECT("BOW",                  "object1.png", 6, 0),
OBJECT("STATUE",               "object1.png", 0, 1),
OBJECT("CHEST",                "object1.png", 1, 1),
OBJECT("GOLD_PIECE",           "object1.png", 2, 1),
OBJECT("SPE_BLANK_PAPER",      "object1.png", 3, 1),
OBJECT("HELMET",               "object1.png", 4, 1),
OBJECT("DWARVISH_ROUNDSHIELD", "object1.png", 5, 1),
OBJECT("AMULET_OF_ESP",        "object1.png", 6, 1),
OBJECT("DAGGER",               "object1.png", 7, 1),
OBJECT("LOW_BOOTS",            "object1.png", 0, 2),
OBJECT("SPEAR",                "object1.png", 1, 2),
OBJECT("SCR_BLANK_PAPER",      "object1.png", 3, 2),
OBJECT("WAN_LIGHT",            "object1.png", 4, 2),
OBJECT("LONG_SWORD",           "object1.png", 5, 2),
OBJECT("RIN_ADORNMENT",        "object1.png", 6, 2),
OBJECT("APPLE",                "object1.png", 7, 2),
OBJECT("GEM_BLUE_GLASS",       "object1.png", 0, 3),
OBJECT("RING_MAIL",            "object1.png", 1, 3),
OBJECT("LEATHER_ARMOR",        "object1.png", 2, 3),
OBJECT("PLATE_MAIL",           "object1.png", 3, 3),
OBJECT("WAR_HAMMER",           "object1.png", 4, 3),
OBJECT("AXE",                  "object1.png", 5, 3),
OBJECT("BRASS_LANTERN",        "object1.png", 6, 3),
OBJECT("PEAR",                 "object1.png", 7, 3),
OBJECT("SCALE_MAIL",           "object1.png", 0, 4),
OBJECT("CHAIN_MAIL",           "object1.png", 1, 4),
OBJECT("ELVEN_CLOAK",          "object1.png", 2, 4),
OBJECT("TRIDENT",              "object1.png", 3, 4),
OBJECT("EXPENSIVE_CAMERA",     "object1.png", 4, 4),
OBJECT("FEDORA",               "object1.png", 5, 4),
OBJECT("CLUB",                 "object1.png", 6, 4),
OBJECT("ARROW",                "object1.png", 7, 4),
OBJECT("EGG",                  "object2.png", 0, 0),
OBJECT("LEATHER_GLOVES",       "object2.png", 1, 0),
OBJECT("BELL",                 "object2.png", 2, 0),
OBJECT("CANDY_BAR",            "object2.png", 6, 0),
OBJECT("WAX_CANDLE",           "object2.png", 5, 0),
OBJECT("CARROT",               "object2.png", 7, 0),
OBJECT("GEM_RED_GLASS",        "object2.png", 3, 0),
OBJECT("GEM_GREEN_GLASS",      "object2.png", 4, 0),
OBJECT("GEM_YELLOW_GLASS",     "object2.png", 0, 1),
OBJECT("GEM_WHITE_GLASS",      "object2.png", 1, 1),
OBJECT("GEM_BLACK_GLASS",      "object2.png", 2, 1),
OBJECT("BULLWHIP",             "object2.png", 3, 1),
OBJECT("MACE",                 "object2.png", 4, 1),
OBJECT("TOOLED_HORN",          "object2.png", 5, 1),
OBJECT("FORTUNE_COOKIE",       "object2.png", 6, 1),
OBJECT("BANANA",               "object2.png", 7, 1),
OBJECT("CRYSTAL_BALL",         "object2.png", 0, 2),
OBJECT("FROST_HORN",           "object2.png", 1, 2),
OBJECT("UNICORN_HORN",         "object2.png", 2, 2),
OBJECT("HAWAIIAN_SHIRT",       "object2.png", 3, 2),
OBJECT("CREDIT_CARD",          "object2.png", 4, 2),
OBJECT("MIRROR",               "object2.png", 5, 2),
OBJECT("CREAM_PIE",            "object2.png", 6, 2),
OBJECT("ORANGE",               "object2.png", 7, 2),
OBJECT("CROSSBOW",             "object2.png", 0, 3),
OBJECT("CORNUTHAUM",           "object2.png", 1, 3),
OBJECT("MAGIC_MARKER",         "object2.png", 2, 3),
OBJECT("QUARTERSTAFF",         "object2.png", 3, 3),
OBJECT("FOOD_RATION",          "object2.png", 4, 3),
OBJECT("PANCAKE",              "object2.png", 5, 3),
OBJECT("LEMBAS_WAFER",         "object2.png", 6, 3),
OBJECT("KELP_FROND",           "object2.png", 7, 3),
OBJECT("TRIPE_RATION",         "object2.png", 0, 4),
OBJECT("MEAT_STICK",           "object2.png", 1, 4),
OBJECT("SHURIKEN",             "object2.png", 2, 4),
OBJECT("PICK_AXE",             "object2.png", 4, 4),
OBJECT("TIN",                  "object2.png", 5, 4),
OBJECT("CLOVE_OF_GARLIC",      "object2.png", 6, 4),
OBJECT("EUCALYPTUS_LEAF",      "object2.png", 7, 4),
OBJECT("MELON",                "object3.png", 0, 0),
OBJECT("SLIME_MOLD",           "object3.png", 1, 0),
OBJECT("SPRIG_OF_WOLFSBANE",   "object3.png", 2, 0),
OBJECT("CRAM_RATION",          "object3.png", 3, 0),
OBJECT("HUGE_CHUNK_OF_MEAT",   "object3.png", 4, 0),
OBJECT("MEATBALL",             "object3.png", 5, 0),
OBJECT("K_RATION",             "object3.png", 6, 0),
OBJECT("C_RATION",             "object3.png", 7, 0),
OBJECT("BOOMERANG",            "object3.png", 0, 1),
OBJECT("LUMP_OF_ROYAL_JELLY",  "object3.png", 1, 1),
OBJECT("MEAT_RING",            "object3.png", 2, 1),
OBJECT("DILITHIUM_CRYSTAL",    "object3.png", 3, 1),
OBJECT("DIAMOND",              "object3.png", 4, 1),
OBJECT("RUBY",                 "object3.png", 5, 1),
OBJECT("JACINTH",              "object3.png", 6, 1),
OBJECT("SAPPHIRE",             "object3.png", 7, 1),
OBJECT("EMERALD",              "object3.png", 0, 2),
OBJECT("BLACK_OPAL",           "object3.png", 1, 2),
OBJECT("TURQUOISE",            "object3.png", 2, 2),
OBJECT("CITRINE",              "object3.png", 3, 2),
OBJECT("AQUAMARINE",           "object3.png", 4, 2),
OBJECT("AMBER",                "object3.png", 5, 2),
OBJECT("TOPAZ",                "object3.png", 6, 2),
OBJECT("JET",                  "object3.png", 7, 2),
OBJECT("OPAL",                 "object3.png", 0, 3),
OBJECT("CHRYSOBERYL",          "object3.png", 1, 3),
OBJECT("GARNET",               "object3.png", 2, 3),
OBJECT("AMETHYST",             "object3.png", 3, 3),
OBJECT("JASPER",               "object3.png", 4, 3),
OBJECT("FLUORITE",             "object3.png", 5, 3),
OBJECT("OBSIDIAN",             "object3.png", 6, 3),
OBJECT("AGATE",                "object3.png", 7, 3),
OBJECT("JADE",                 "object3.png", 0, 4),
OBJECT("LUCKSTONE",            "object3.png", 1, 4),
OBJECT("LOADSTONE",            "object3.png", 2, 4),
OBJECT("FLINT",                "object3.png", 3, 4),
OBJECT("GEM_VIOLET_GLASS",     "object3.png", 4, 4),
OBJECT("GEM_ORANGE_GLASS",     "object3.png", 5, 4),
OBJECT("TINNING_KIT",          "object3.png", 6, 4),
OBJECT("FIRE_HORN",            "object3.png", 7, 4),
OBJECT("POT_GAIN_ABILITY",     "object4.png", 0, 0),
OBJECT("POT_RESTORE_ABILITY",  "object4.png", 1, 0),
OBJECT("POT_CONFUSION",        "object4.png", 2, 0),
OBJECT("POT_BLINDNESS",        "object4.png", 3, 0),
OBJECT("POT_PARALYSIS",        "object4.png", 4, 0),
OBJECT("POT_SPEED",            "object4.png", 5, 0),
OBJECT("POT_LEVITATION",       "object4.png", 6, 0),
OBJECT("POT_HALLUCINATION",    "object4.png", 7, 0),
OBJECT("POT_INVISIBILITY",     "object4.png", 0, 1),
OBJECT("POT_SEE_INVISIBLE",    "object4.png", 1, 1),
OBJECT("POT_HEALING",          "object4.png", 2, 1),
OBJECT("POT_EXTRA_HEALING",    "object4.png", 3, 1),
OBJECT("POT_GAIN_LEVEL",       "object4.png", 4, 1),
OBJECT("POT_ENLIGHTENMENT",    "object4.png", 5, 1),
OBJECT("POT_MONSTER_DETECTION","object4.png", 6, 1),
OBJECT("POT_OBJECT_DETECTION", "object4.png", 7, 1),
OBJECT("POT_GAIN_ENERGY",      "object4.png", 0, 2),
OBJECT("POT_SLEEPING",         "object4.png", 1, 2),
OBJECT("POT_FULL_HEALING",     "object4.png", 2, 2),
OBJECT("POT_POLYMORPH",        "object4.png", 3, 2),
OBJECT("POT_BOOZE",            "object4.png", 4, 2),
OBJECT("POT_SICKNESS",         "object4.png", 5, 2),
OBJECT("POT_FRUIT_JUICE",      "object4.png", 6, 2),
OBJECT("POT_ACID",             "object4.png", 7, 2),
OBJECT("POT_OIL",              "object4.png", 0, 3),
OBJECT("POT_WATER",            "object4.png", 1, 3),
OBJECT("SACK",                 "object4.png", 4, 3),
OBJECT("BAG_OF_HOLDING",       "object4.png", 5, 3),
OBJECT("OILSKIN_SACK",         "object4.png", 6, 3),
OBJECT("BAG_OF_TRICKS",        "object4.png", 7, 3),
OBJECT("ICE_BOX",              "object4.png", 0, 4),
OBJECT("LARGE_BOX",            "object4.png", 1, 4),
OBJECT("TOUCHSTONE",           "object4.png", 3, 4),
OBJECT("ROCK",                 "object4.png", 4, 4),
OBJECT("HEALTHSTONE",          "object4.png", 6, 4),
OBJECT("WHETSTONE",            "object4.png", 7, 4),
OBJECT("GEM_BROWN_GLASS",      "object4.png", 5, 4),
OBJECT("LEASH",                "object5.png", 0, 0),
OBJECT("LENSES",               "object5.png", 1, 0),
OBJECT("BLINDFOLD",            "object5.png", 2, 0),
OBJECT("TOWEL",                "object5.png", 3, 0),
OBJECT("STETHOSCOPE",          "object5.png", 4, 0),
OBJECT("MAGIC_WHISTLE",        "object5.png", 5, 0),
OBJECT("TIN_WHISTLE",          "object5.png", 6, 0),
OBJECT("SADDLE",               "object5.png", 7, 0),
OBJECT("WOODEN_FLUTE",         "object5.png", 0, 1),
OBJECT("WOODEN_HARP",          "object5.png", 1, 1),
OBJECT("CANDELABRUM_OF_INVOCATION","object5.png", 2, 1),
OBJECT("IRON_CHAIN",           "object5.png", 3, 1),
OBJECT("HEAVY_IRON_BALL",      "object5.png", 4, 1),
OBJECT("LEATHER_DRUM",         "object5.png", 5, 1),
OBJECT("FIGURINE",             "object5.png", 6, 1),
OBJECT("BEARTRAP",             "object5.png", 0, 2),
OBJECT("T_SHIRT",              "object5.png", 1, 2),
OBJECT("TIN_OPENER",           "object5.png", 2, 2),
OBJECT("BUGLE",                "object5.png", 3, 2),
OBJECT("CRYSTAL_PLATE_MAIL",   "object5.png", 4, 2),
OBJECT("BRONZE_PLATE_MAIL",    "object5.png", 5, 2),
OBJECT("SCR_ENCHANT_ARMOR",    "object5.png", 6, 2),
OBJECT("SCR_DESTROY_ARMOR",    "object5.png", 7, 2),
OBJECT("SCR_CONFUSE_MONSTER",  "object5.png", 0, 3),
OBJECT("SCR_SCARE_MONSTER",    "object5.png", 1, 3),
OBJECT("SCR_REMOVE_CURSE",     "object5.png", 2, 3),
OBJECT("SCR_ENCHANT_WEAPON",   "object5.png", 3, 3),
OBJECT("SCR_CREATE_MONSTER",   "object5.png", 4, 3),
OBJECT("SCR_TAMING",           "object5.png", 5, 3),
OBJECT("SCR_GENOCIDE",         "object5.png", 6, 3),
OBJECT("SCR_LIGHT",            "object5.png", 7, 3),
OBJECT("SCR_TELEPORTATION",    "object5.png", 0, 4),
OBJECT("SCR_GOLD_DETECTION",   "object5.png", 1, 4),
OBJECT("SCR_FOOD_DETECTION",   "object5.png", 2, 4),
OBJECT("SCR_IDENTIFY",         "object5.png", 3, 4),
OBJECT("SCR_MAGIC_MAPPING",    "object5.png", 4, 4),
OBJECT("SCR_AMNESIA",          "object5.png", 5, 4),
OBJECT("SCR_FIRE",             "object5.png", 6, 4),
OBJECT("SCR_EARTH",            "object5.png", 7, 4),
OBJECT("SCR_PUNISHMENT",       "object6.png", 0, 0),
OBJECT("SCR_CHARGING",         "object6.png", 1, 0),
OBJECT("SCR_STINKING_CLOUD",   "object6.png", 2, 0),
OBJECT("SCR_UNNAMED_1",        "object6.png", 3, 0),
OBJECT("SCR_UNNAMED_2",        "object6.png", 4, 0),
OBJECT("SCR_UNNAMED_3",        "object6.png", 5, 0),
OBJECT("SCR_UNNAMED_4",        "object6.png", 6, 0),
OBJECT("SCR_MAIL",             "object6.png", 7, 0),
OBJECT("CAN_OF_GREASE",        "object6.png", 0, 1),
OBJECT("MAGIC_LAMP",           "object6.png", 1, 1),
OBJECT("OIL_LAMP",             "object6.png", 2, 1),
OBJECT("KATANA",               "object6.png", 3, 1),
OBJECT("SHIELD_OF_REFLECTION", "object6.png", 4, 1),
OBJECT("ELVEN_SHIELD",         "object6.png", 5, 1),
OBJECT("SILVER_DRAGON_SCALES", "object6.png", 6, 1),
OBJECT("BLACK_DRAGON_SCALES",  "object6.png", 7, 1),
OBJECT("GRAY_DRAGON_SCALES",   "object6.png", 0, 2),
OBJECT("WHITE_DRAGON_SCALES",  "object6.png", 1, 2),
OBJECT("YELLOW_DRAGON_SCALES", "object6.png", 2, 2),
OBJECT("ORANGE_DRAGON_SCALES", "object6.png", 3, 2),
OBJECT("GREEN_DRAGON_SCALES",  "object6.png", 4, 2),
OBJECT("RED_DRAGON_SCALES",    "object6.png", 5, 2),
OBJECT("BLUE_DRAGON_SCALES",   "object6.png", 6, 2),
OBJECT("SPE_BOOK_OF_THE_DEAD", "object6.png", 7, 2),
OBJECT("SPE_PROTECTION",       "object6.png", 0, 3),
OBJECT("SPE_CLAIRVOYANCE",     "object6.png", 1, 3),
OBJECT("SPE_CURE_SICKNESS",    "object6.png", 2, 3),
OBJECT("SPE_DETECT_FOOD",      "object6.png", 3, 3),
OBJECT("SPE_CAUSE_FEAR",       "object6.png", 4, 3),
OBJECT("SPE_CREATE_MONSTER",   "object6.png", 5, 3),
OBJECT("SPE_JUMPING",          "object6.png", 6, 3),
OBJECT("SPE_SLOW_MONSTER",     "object6.png", 7, 3),
OBJECT("SPE_DIG",              "object6.png", 0, 4),
OBJECT("SPE_WIZARD_LOCK",      "object6.png", 1, 4),
OBJECT("SPE_MAGIC_MISSILE",    "object6.png", 2, 4),
OBJECT("SPE_RESTORE_ABILITY",  "object6.png", 3, 4),
OBJECT("SPE_INVISIBILITY",     "object6.png", 4, 4),
OBJECT("SPE_CONFUSE_MONSTER",  "object6.png", 5, 4),
OBJECT("SPE_LEVITATION",       "object6.png", 6, 4),
OBJECT("SPE_CHARM_MONSTER",    "object6.png", 7, 4),
OBJECT("SPE_FORCE_BOLT",       "object7.png", 0, 0),
OBJECT("SPE_HASTE_SELF",       "object7.png", 1, 0),
OBJECT("SPE_DETECT_UNSEEN",    "object7.png", 2, 0),
OBJECT("SPE_KNOCK",            "object7.png", 3, 0),
OBJECT("SPE_FIREBALL",         "object7.png", 4, 0),
OBJECT("SPE_CURE_BLINDNESS",   "object7.png", 5, 0),
OBJECT("SPE_TELEPORT_AWAY",    "object7.png", 6, 0),
OBJECT("SPE_DETECT_TREASURE",  "object7.png", 7, 0),
OBJECT("SPE_MAGIC_MAPPING",    "object7.png", 0, 1),
OBJECT("SPE_CANCELLATION",     "object7.png", 1, 1),
OBJECT("SPE_HEALING",          "object7.png", 2, 1),
OBJECT("SPE_CONE_OF_COLD",     "object7.png", 3, 1),
OBJECT("SPE_SLEEP",            "object7.png", 4, 1),
OBJECT("SPE_FINGER_OF_DEATH",  "object7.png", 5, 1),
OBJECT("SPE_DRAIN_LIFE",       "object7.png", 6, 1),
OBJECT("SPE_DETECT_MONSTERS",  "object7.png", 7, 1),
OBJECT("SPE_LIGHT",            "object7.png", 0, 2),
OBJECT("SPE_EXTRA_HEALING",    "object7.png", 1, 2),
OBJECT("SPE_REMOVE_CURSE",     "object7.png", 2, 2),
OBJECT("SPE_TURN_UNDEAD",      "object7.png", 3, 2),
OBJECT("SPE_POLYMORPH",        "object7.png", 4, 2),
OBJECT("SPE_IDENTIFY",         "object7.png", 5, 2),
OBJECT("SPE_STONE_TO_FLESH",   "object7.png", 6, 2),
OBJECT("SPE_CREATE_FAMILIAR",  "object7.png", 7, 2),
OBJECT("BELL_OF_OPENING",      "object7.png", 0, 3),
OBJECT("ELVEN_ARROW",          "object7.png", 1, 3),
OBJECT("SILVER_ARROW",         "object7.png", 2, 3),
OBJECT("ORCISH_ARROW",         "object7.png", 3, 3),
OBJECT("YA",                   "object7.png", 4, 3),
OBJECT("CROSSBOW_BOLT",        "object7.png", 5, 3),
OBJECT("DART",                 "object7.png", 6, 3),
OBJECT("SLING",                "object7.png", 7, 3),
OBJECT("ORCISH_SHIELD",        "object7.png", 0, 4),
OBJECT("SMALL_SHIELD",         "object7.png", 1, 4),
OBJECT("URUK_HAI_SHIELD",      "object7.png", 2, 4),
OBJECT("LARGE_SHIELD",         "object7.png", 3, 4),
OBJECT("GRAPPLING_HOOK",       "object7.png", 4, 4),


/* figurines:NAME                   FILENAME          top left x coord     top left y coord    width       height*/
/* The name field is the name of a define in pm.h, _as a string_ ! */
FIGURINE("PM_FREEZING_SPHERE",   "figurine1.png", 0, 0, FIG_DX, FIG_DY),
FIGURINE("PM_FLAMING_SPHERE",    "figurine1.png", 1, 0, FIG_DX, FIG_DY),
FIGURINE("PM_SHOCKING_SPHERE",   "figurine1.png", 2, 0, FIG_DX, FIG_DY),
FIGURINE("PM_PONY",              "figurine1.png", 3, 0, FIG_DX, FIG_DY),
FIGURINE("PM_HORSE",             "figurine1.png", 4, 0, FIG_DX, FIG_DY),
FIGURINE("PM_WARHORSE",          "figurine1.png", 5, 0, FIG_DX, FIG_DY),
FIGURINE("PM_ACID_BLOB",         "figurine1.png", 6, 0, FIG_DX, FIG_DY),
FIGURINE("PM_QUIVERING_BLOB",    "figurine1.png", 7, 0, FIG_DX, FIG_DY),
FIGURINE("PM_GELATINOUS_CUBE",   "figurine1.png", 8, 0, FIG_DX, FIG_DY),
FIGURINE("PM_BROWN_MOLD",        "figurine1.png", 9, 0, FIG_DX, FIG_DY),
FIGURINE("PM_LICHEN",            "figurine1.png", 10, 0, FIG_DX, FIG_DY),
FIGURINE("PM_GREEN_MOLD",        "figurine1.png", 11, 0, FIG_DX, FIG_DY),
FIGURINE("PM_ROTHE",             "figurine1.png", 0, 1, FIG_DX, FIG_DY),
FIGURINE("PM_GNOME",             "figurine1.png", 1, 1, FIG_DX, FIG_DY),
FIGURINE("PM_GREEN_ELF",         "figurine1.png", 2, 1, FIG_DX, FIG_DY),
FIGURINE("PM_DWARF",             "figurine1.png", 3, 1, FIG_DX, FIG_DY),
FIGURINE("PM_WATER_MOCCASIN",    "figurine1.png", 4, 1, FIG_DX, FIG_DY),
FIGURINE("PM_MANES",             "figurine1.png", 5, 1, FIG_DX, FIG_DY),
FIGURINE("PM_SHRIEKER",          "figurine1.png", 6, 1, FIG_DX, FIG_DY),
FIGURINE("PM_STALKER",           "figurine1.png", 7, 1, FIG_DX, FIG_DY),
FIGURINE("PM_AIR_ELEMENTAL",     "figurine1.png", 8, 1, FIG_DX, FIG_DY),
FIGURINE("PM_EARTH_ELEMENTAL",   "figurine1.png", 9, 1, FIG_DX, FIG_DY),
FIGURINE("PM_WATER_ELEMENTAL",   "figurine1.png", 10, 1, FIG_DX, FIG_DY),
FIGURINE("PM_BLACK_LIGHT",       "figurine1.png", 11, 1, FIG_DX, FIG_DY),
FIGURINE("PM_HOBBIT",            "figurine1.png", 0, 2, FIG_DX, FIG_DY),
FIGURINE("PM_BLACK_UNICORN",     "figurine1.png", 1, 2, FIG_DX, FIG_DY),
FIGURINE("PM_GRAY_UNICORN",      "figurine1.png", 2, 2, FIG_DX, FIG_DY),
FIGURINE("PM_WHITE_UNICORN",     "figurine1.png", 3, 2, FIG_DX, FIG_DY),
FIGURINE("PM_GIANT_ANT",         "figurine1.png", 4, 2, FIG_DX, FIG_DY),
FIGURINE("PM_FIRE_ANT",          "figurine1.png", 5, 2, FIG_DX, FIG_DY),
FIGURINE("PM_ROCK_PIERCER",      "figurine1.png", 6, 2, FIG_DX, FIG_DY),
FIGURINE("PM_IRON_PIERCER",      "figurine1.png", 7, 2, FIG_DX, FIG_DY),
FIGURINE("PM_GLASS_PIERCER",     "figurine1.png", 8, 2, FIG_DX, FIG_DY),
FIGURINE("PM_DOG",               "figurine1.png", 9, 2, FIG_DX, FIG_DY),
FIGURINE("PM_LARGE_DOG",         "figurine1.png", 10, 2, FIG_DX, FIG_DY),
FIGURINE("PM_WATER_DEMON",       "figurine1.png", 11, 2, FIG_DX, FIG_DY),
FIGURINE("PM_ELVENKING",         "figurine1.png", 0, 3, FIG_DX, FIG_DY),
FIGURINE("PM_ELF_LORD",          "figurine1.png", 1, 3, FIG_DX, FIG_DY),
FIGURINE("PM_GREY_ELF",          "figurine1.png", 2, 3, FIG_DX, FIG_DY),
FIGURINE("PM_WOODLAND_ELF",      "figurine1.png", 3, 3, FIG_DX, FIG_DY),
FIGURINE("PM_ELF",               "figurine1.png", 4, 3, FIG_DX, FIG_DY),
FIGURINE("PM_ROCK_MOLE",         "figurine1.png", 5, 3, FIG_DX, FIG_DY),
FIGURINE("PM_GECKO",             "figurine1.png", 6, 3, FIG_DX, FIG_DY),
FIGURINE("PM_NEWT",              "figurine1.png", 7, 3, FIG_DX, FIG_DY),
FIGURINE("PM_GAS_SPORE",         "figurine1.png", 9, 3, FIG_DX, FIG_DY),
FIGURINE("PM_KITTEN",            "figurine1.png", 10, 3, FIG_DX, FIG_DY),
FIGURINE("PM_LARGE_CAT",         "figurine1.png", 11, 3, FIG_DX, FIG_DY),
FIGURINE("PM_DWARF_LORD",        "figurine1.png", 0, 4, FIG_DX, FIG_DY),
FIGURINE("PM_DWARF_KING",        "figurine1.png", 1, 4, FIG_DX, FIG_DY),
FIGURINE("PM_GNOME_LORD",        "figurine1.png", 2, 4, FIG_DX, FIG_DY),
FIGURINE("PM_GNOME_KING",        "figurine1.png", 3, 4, FIG_DX, FIG_DY),
FIGURINE("PM_GNOMISH_WIZARD",    "figurine1.png", 4, 4, FIG_DX, FIG_DY),
FIGURINE("PM_ORC",               "figurine1.png", 5, 4, FIG_DX, FIG_DY),
FIGURINE("PM_KOBOLD_ZOMBIE",     "figurine1.png", 6, 4, FIG_DX, FIG_DY),
FIGURINE("PM_GNOME_ZOMBIE",      "figurine1.png", 7, 4, FIG_DX, FIG_DY),
FIGURINE("PM_DWARF_ZOMBIE",      "figurine1.png", 8, 4, FIG_DX, FIG_DY),
FIGURINE("PM_ORC_ZOMBIE",        "figurine1.png", 9, 4, FIG_DX, FIG_DY),
FIGURINE("PM_ELF_ZOMBIE",        "figurine1.png", 10, 4, FIG_DX, FIG_DY),
FIGURINE("PM_KOBOLD_MUMMY",      "figurine1.png", 11, 4, FIG_DX, FIG_DY),
FIGURINE("PM_HILL_ORC",          "figurine1.png", 0, 5, FIG_DX, FIG_DY),
FIGURINE("PM_MORDOR_ORC",        "figurine1.png", 1, 5, FIG_DX, FIG_DY),
FIGURINE("PM_URUK_HAI",          "figurine1.png", 2, 5, FIG_DX, FIG_DY),
FIGURINE("PM_HOBGOBLIN",         "figurine1.png", 3, 5, FIG_DX, FIG_DY),
FIGURINE("PM_BUGBEAR",           "figurine1.png", 4, 5, FIG_DX, FIG_DY),
FIGURINE("PM_KOBOLD",            "figurine1.png", 5, 5, FIG_DX, FIG_DY),
FIGURINE("PM_GNOME_MUMMY",       "figurine1.png", 6, 5, FIG_DX, FIG_DY),
FIGURINE("PM_DWARF_MUMMY",       "figurine1.png", 7, 5, FIG_DX, FIG_DY),
FIGURINE("PM_ORC_MUMMY",         "figurine1.png", 8, 5, FIG_DX, FIG_DY),
FIGURINE("PM_HUMAN_MUMMY",       "figurine1.png", 9, 5, FIG_DX, FIG_DY),
FIGURINE("PM_ELF_MUMMY",         "figurine1.png", 10, 5, FIG_DX, FIG_DY),
FIGURINE("PM_HOMUNCULUS",        "figurine1.png", 11, 5, FIG_DX, FIG_DY),
FIGURINE("PM_LARGE_KOBOLD",      "figurine1.png", 0, 6, FIG_DX, FIG_DY),
FIGURINE("PM_KOBOLD_SHAMAN",     "figurine1.png", 1, 6, FIG_DX, FIG_DY),
FIGURINE("PM_KOBOLD_LORD",       "figurine1.png", 2, 6, FIG_DX, FIG_DY),
FIGURINE("PM_ORC_CAPTAIN",       "figurine1.png", 3, 6, FIG_DX, FIG_DY),
FIGURINE("PM_ORC_SHAMAN",        "figurine1.png", 4, 6, FIG_DX, FIG_DY),
FIGURINE("PM_LEPRECHAUN",        "figurine1.png", 5, 6, FIG_DX, FIG_DY),
FIGURINE("PM_IMP",               "figurine1.png", 6, 6, FIG_DX, FIG_DY),
FIGURINE("PM_LEMURE",            "figurine1.png", 7, 6, FIG_DX, FIG_DY),
FIGURINE("PM_QUASIT",            "figurine1.png", 8, 6, FIG_DX, FIG_DY),
FIGURINE("PM_TENGU",             "figurine1.png", 9, 6, FIG_DX, FIG_DY),
FIGURINE("PM_ORACLE",            "figurine1.png", 10, 6, FIG_DX, FIG_DY),
FIGURINE("PM_ALIGNED_PRIEST",    "figurine1.png", 11, 6, FIG_DX, FIG_DY),
FIGURINE("PM_GARTER_SNAKE",      "figurine1.png", 0, 7, FIG_DX, FIG_DY),
FIGURINE("PM_SNAKE",             "figurine1.png", 1, 7, FIG_DX, FIG_DY),
FIGURINE("PM_PIT_VIPER",         "figurine1.png", 2, 7, FIG_DX, FIG_DY),
FIGURINE("PM_PYTHON",            "figurine1.png", 3, 7, FIG_DX, FIG_DY),
FIGURINE("PM_COBRA",             "figurine1.png", 4, 7, FIG_DX, FIG_DY),
FIGURINE("PM_WUMPUS",            "figurine1.png", 5, 7, FIG_DX, FIG_DY),
FIGURINE("PM_WATCHMAN",          "figurine1.png", 9, 7, FIG_DX, FIG_DY),
FIGURINE("PM_WATCH_CAPTAIN",     "figurine1.png", 10, 7, FIG_DX, FIG_DY),
FIGURINE("PM_SOLDIER",           "figurine1.png", 11, 7, FIG_DX, FIG_DY),
FIGURINE("PM_SERGEANT",          "figurine2.png", 0, 0, FIG_DX, FIG_DY),
FIGURINE("PM_LIEUTENANT",        "figurine2.png", 1, 0, FIG_DX, FIG_DY),
FIGURINE("PM_CAPTAIN",           "figurine2.png", 2, 0, FIG_DX, FIG_DY),
FIGURINE("PM_GRID_BUG",          "figurine2.png", 3, 0, FIG_DX, FIG_DY),
FIGURINE("PM_XAN",               "figurine2.png", 4, 0, FIG_DX, FIG_DY),
FIGURINE("PM_GARGOYLE",          "figurine2.png", 5, 0, FIG_DX, FIG_DY),
FIGURINE("PM_ARCH_LICH",         "figurine2.png", 6, 0, FIG_DX, FIG_DY),
FIGURINE("PM_GUARDIAN_NAGA",     "figurine2.png", 7, 0, FIG_DX, FIG_DY),
FIGURINE("PM_GUARDIAN_NAGA_HATCHLING","figurine2.png", 8, 0, FIG_DX, FIG_DY),
FIGURINE("PM_GOLDEN_NAGA",       "figurine2.png", 9, 0, FIG_DX, FIG_DY),
FIGURINE("PM_GOLDEN_NAGA_HATCHLING","figurine2.png", 10, 0, FIG_DX, FIG_DY),
FIGURINE("PM_RED_NAGA",          "figurine2.png", 11, 0, FIG_DX, FIG_DY),
FIGURINE("PM_WINGED_GARGOYLE",   "figurine2.png", 0, 1, FIG_DX, FIG_DY),
FIGURINE("PM_DINGO",             "figurine2.png", 1, 1, FIG_DX, FIG_DY),
FIGURINE("PM_WOLF",              "figurine2.png", 2, 1, FIG_DX, FIG_DY),
FIGURINE("PM_WEREWOLF",          "figurine2.png", 3, 1, FIG_DX, FIG_DY),
FIGURINE("PM_WARG",              "figurine2.png", 4, 1, FIG_DX, FIG_DY),
FIGURINE("PM_WINTER_WOLF_CUB",   "figurine2.png", 5, 1, FIG_DX, FIG_DY),
FIGURINE("PM_RED_NAGA_HATCHLING","figurine2.png", 6, 1, FIG_DX, FIG_DY),
FIGURINE("PM_BLACK_NAGA",        "figurine2.png", 7, 1, FIG_DX, FIG_DY),
FIGURINE("PM_BLACK_NAGA_HATCHLING","figurine2.png", 8, 1, FIG_DX, FIG_DY),
FIGURINE("PM_RUST_MONSTER",      "figurine2.png", 9, 1, FIG_DX, FIG_DY),
FIGURINE("PM_GREMLIN",           "figurine2.png", 10, 1, FIG_DX, FIG_DY),
FIGURINE("PM_BABY_PURPLE_WORM",  "figurine2.png", 11, 1, FIG_DX, FIG_DY),
FIGURINE("PM_WINTER_WOLF",       "figurine2.png", 0, 2, FIG_DX, FIG_DY),
FIGURINE("PM_HELL_HOUND_PUP",    "figurine2.png", 1, 2, FIG_DX, FIG_DY),
FIGURINE("PM_HELL_HOUND",        "figurine2.png", 2, 2, FIG_DX, FIG_DY),
FIGURINE("PM_JAGUAR",            "figurine2.png", 3, 2, FIG_DX, FIG_DY),
FIGURINE("PM_LYNX",              "figurine2.png", 4, 2, FIG_DX, FIG_DY),
FIGURINE("PM_PANTHER",           "figurine2.png", 5, 2, FIG_DX, FIG_DY),
FIGURINE("PM_MONKEY",            "figurine2.png", 6, 2, FIG_DX, FIG_DY),
FIGURINE("PM_APE",               "figurine2.png", 7, 2, FIG_DX, FIG_DY),
FIGURINE("PM_OWLBEAR",           "figurine2.png", 8, 2, FIG_DX, FIG_DY),
FIGURINE("PM_YETI",              "figurine2.png", 9, 2, FIG_DX, FIG_DY),
FIGURINE("PM_CARNIVOROUS_APE",   "figurine2.png", 10, 2, FIG_DX, FIG_DY),
FIGURINE("PM_STRAW_GOLEM",       "figurine2.png", 11, 2, FIG_DX, FIG_DY),
FIGURINE("PM_TIGER",             "figurine2.png", 0, 3, FIG_DX, FIG_DY),
FIGURINE("PM_MIND_FLAYER",       "figurine2.png", 1, 3, FIG_DX, FIG_DY),
FIGURINE("PM_MASTER_MIND_FLAYER","figurine2.png", 2, 3, FIG_DX, FIG_DY),
FIGURINE("PM_LICH",              "figurine2.png", 3, 3, FIG_DX, FIG_DY),
FIGURINE("PM_DEMILICH",          "figurine2.png", 4, 3, FIG_DX, FIG_DY),
FIGURINE("PM_MASTER_LICH",       "figurine2.png", 5, 3, FIG_DX, FIG_DY),
FIGURINE("PM_PAPER_GOLEM",       "figurine2.png", 6, 3, FIG_DX, FIG_DY),
FIGURINE("PM_ROPE_GOLEM",        "figurine2.png", 7, 3, FIG_DX, FIG_DY),
FIGURINE("PM_GOLD_GOLEM",        "figurine2.png", 8, 3, FIG_DX, FIG_DY),
FIGURINE("PM_LEATHER_GOLEM",     "figurine2.png", 9, 3, FIG_DX, FIG_DY),
FIGURINE("PM_WOOD_GOLEM",        "figurine2.png", 10, 3, FIG_DX, FIG_DY),
FIGURINE("PM_FLESH_GOLEM",       "figurine2.png", 11, 3, FIG_DX, FIG_DY),
FIGURINE("PM_ARCHEOLOGIST",      "figurine2.png", 0, 4, FIG_DX, FIG_DY),
FIGURINE("PM_TOURIST",           "figurine2.png", 1, 4, FIG_DX, FIG_DY),
FIGURINE("PM_ROGUE",             "figurine2.png", 2, 4, FIG_DX, FIG_DY),
FIGURINE("PM_PRIEST",            "figurine2.png", 3, 4, FIG_DX, FIG_DY),
FIGURINE("PM_COYOTE",            "figurine2.png", 4, 4, FIG_DX, FIG_DY),
FIGURINE("PM_JACKAL",            "figurine2.png", 5, 4, FIG_DX, FIG_DY),
FIGURINE("PM_KNIGHT",            "figurine2.png", 6, 4, FIG_DX, FIG_DY),
FIGURINE("PM_GOBLIN",            "figurine2.png", 7, 4, FIG_DX, FIG_DY),
FIGURINE("PM_QUEEN_BEE",         "figurine2.png", 8, 4, FIG_DX, FIG_DY),
FIGURINE("PM_FIRE_ELEMENTAL",    "figurine2.png", 9, 4, FIG_DX, FIG_DY),
FIGURINE("PM_GREEN_SLIME",       "figurine2.png", 10, 4, FIG_DX, FIG_DY),
FIGURINE("PM_SKELETON",          "figurine2.png", 11, 4, FIG_DX, FIG_DY),
FIGURINE("PM_VAMPIRE_BAT",       "figurine2.png", 0, 5, FIG_DX, FIG_DY),
FIGURINE("PM_FOREST_CENTAUR",    "figurine2.png", 1, 5, FIG_DX, FIG_DY),
FIGURINE("PM_SOLDIER_ANT",       "figurine2.png", 2, 5, FIG_DX, FIG_DY),
FIGURINE("PM_KILLER_BEE",        "figurine2.png", 3, 5, FIG_DX, FIG_DY),
FIGURINE("PM_YELLOW_LIGHT",      "figurine2.png", 4, 5, FIG_DX, FIG_DY),
FIGURINE("PM_FOX",               "figurine2.png", 5, 5, FIG_DX, FIG_DY),
FIGURINE("PM_FLOATING_EYE",      "figurine2.png", 6, 5, FIG_DX, FIG_DY),
FIGURINE("PM_QUANTUM_MECHANIC",  "figurine2.png", 7, 5, FIG_DX, FIG_DY),
FIGURINE("PM_HOUSECAT",          "figurine2.png", 8, 5, FIG_DX, FIG_DY),
FIGURINE("PM_LIZARD",            "figurine2.png", 9, 5, FIG_DX, FIG_DY),
FIGURINE("PM_LITTLE_DOG",        "figurine2.png", 10, 5, FIG_DX, FIG_DY),
FIGURINE("PM_TROLL",             "figurine2.png", 11, 5, FIG_DX, FIG_DY),
FIGURINE("PM_COCKATRICE",        "figurine2.png", 0, 6, FIG_DX, FIG_DY),
FIGURINE("PM_KRAKEN",            "figurine2.png", 1, 6, FIG_DX, FIG_DY),
FIGURINE("PM_GIANT_EEL",         "figurine2.png", 2, 6, FIG_DX, FIG_DY),
FIGURINE("PM_PYROLISK",          "figurine2.png", 3, 6, FIG_DX, FIG_DY),
FIGURINE("PM_YELLOW_MOLD",       "figurine2.png", 4, 6, FIG_DX, FIG_DY),
FIGURINE("PM_SHARK",             "figurine2.png", 5, 6, FIG_DX, FIG_DY),
FIGURINE("PM_WOOD_NYMPH",        "figurine2.png", 6, 6, FIG_DX, FIG_DY),
FIGURINE("PM_MOUNTAIN_NYMPH",    "figurine2.png", 7, 6, FIG_DX, FIG_DY),
FIGURINE("PM_WIZARD",            "figurine2.png", 8, 6, FIG_DX, FIG_DY),
FIGURINE("PM_VALKYRIE",          "figurine2.png", 9, 6, FIG_DX, FIG_DY),
FIGURINE("PM_RANGER",            "figurine2.png", 10, 6, FIG_DX, FIG_DY),
FIGURINE("PM_WATER_NYMPH",       "figurine2.png", 11, 6, FIG_DX, FIG_DY),
FIGURINE("PM_CHICKATRICE",       "figurine2.png", 0, 7, FIG_DX, FIG_DY),
FIGURINE("PM_SMALL_MIMIC",       "figurine2.png", 1, 7, FIG_DX, FIG_DY),
FIGURINE("PM_LARGE_MIMIC",       "figurine2.png", 2, 7, FIG_DX, FIG_DY),
FIGURINE("PM_GIANT_MIMIC",       "figurine2.png", 3, 7, FIG_DX, FIG_DY),
FIGURINE("PM_RED_MOLD",          "figurine2.png", 4, 7, FIG_DX, FIG_DY),
FIGURINE("PM_VIOLET_FUNGUS",     "figurine2.png", 5, 7, FIG_DX, FIG_DY),
FIGURINE("PM_GHOST",             "figurine2.png", 6, 7, FIG_DX, FIG_DY),
FIGURINE("PM_WRAITH",            "figurine2.png", 7, 7, FIG_DX, FIG_DY),
FIGURINE("PM_GIANT_BEETLE",      "figurine2.png", 8, 7, FIG_DX, FIG_DY),
FIGURINE("PM_HUMAN_ZOMBIE",      "figurine2.png", 9, 7, FIG_DX, FIG_DY),
FIGURINE("PM_CENTIPEDE",         "figurine2.png", 10, 7, FIG_DX, FIG_DY),
FIGURINE("PM_GIANT_RAT",         "figurine2.png", 11, 7, FIG_DX, FIG_DY),
FIGURINE("PM_SCORPION",          "figurine3.png", 0, 0, FIG_DX, FIG_DY),
FIGURINE("PM_CAVE_SPIDER",       "figurine3.png", 1, 0, FIG_DX, FIG_DY),
FIGURINE("PM_GIANT_SPIDER",      "figurine3.png", 2, 0, FIG_DX, FIG_DY),
FIGURINE("PM_BAT",               "figurine3.png", 3, 0, FIG_DX, FIG_DY),
FIGURINE("PM_GIANT_BAT",         "figurine3.png", 4, 0, FIG_DX, FIG_DY),
FIGURINE("PM_RAVEN",             "figurine3.png", 5, 0, FIG_DX, FIG_DY),
FIGURINE("PM_DJINNI",            "figurine3.png", 0, 1, FIG_DX, FIG_DY),
FIGURINE("PM_MOUNTAIN_CENTAUR",  "figurine3.png", 1, 1, FIG_DX, FIG_DY),
FIGURINE("PM_PLAINS_CENTAUR",    "figurine3.png", 2, 1, FIG_DX, FIG_DY),
FIGURINE("PM_COUATL",            "figurine3.png", 3, 1, FIG_DX, FIG_DY),
FIGURINE("PM_DOPPELGANGER",      "figurine3.png", 4, 1, FIG_DX, FIG_DY),
FIGURINE("PM_CROESUS",           "figurine3.png", 5, 1, FIG_DX, FIG_DY),
FIGURINE("PM_KEYSTONE_KOP",      "figurine3.png", 0, 2, FIG_DX, FIG_DY),
FIGURINE("PM_KOP_SERGEANT",      "figurine3.png", 1, 2, FIG_DX, FIG_DY),
FIGURINE("PM_KOP_LIEUTENANT",    "figurine3.png", 2, 2, FIG_DX, FIG_DY),
FIGURINE("PM_KOP_KAPTAIN",       "figurine3.png", 3, 2, FIG_DX, FIG_DY),
FIGURINE("PM_VAMPIRE",           "figurine3.png", 4, 2, FIG_DX, FIG_DY),
FIGURINE("PM_SEWER_RAT",         "figurine3.png", 5, 2, FIG_DX, FIG_DY),
FIGURINE("PM_RABID_RAT",         "figurine3.png", 0, 3, FIG_DX, FIG_DY),
FIGURINE("PM_WERERAT",           "figurine3.png", 1, 3, FIG_DX, FIG_DY),
FIGURINE("PM_WOODCHUCK",         "figurine3.png", 2, 3, FIG_DX, FIG_DY),
FIGURINE("PM_MEDUSA",            "figurine3.png", 3, 3, FIG_DX, FIG_DY),
FIGURINE("PM_HIGH_PRIEST",       "figurine3.png", 4, 3, FIG_DX, FIG_DY),
FIGURINE("PM_NURSE",             "figurine3.png", 5, 3, FIG_DX, FIG_DY),
FIGURINE("PM_GIANT",             "figurine4.png", 0, 0, FIG_DX, FIG_LARGE_DY),
FIGURINE("PM_STONE_GIANT",       "figurine4.png", 1, 0, FIG_DX, FIG_LARGE_DY),
FIGURINE("PM_HILL_GIANT",        "figurine4.png", 2, 0, FIG_DX, FIG_LARGE_DY),
FIGURINE("PM_FROST_GIANT",       "figurine4.png", 3, 0, FIG_DX, FIG_LARGE_DY),
FIGURINE("PM_FIRE_GIANT",        "figurine4.png", 4, 0, FIG_DX, FIG_LARGE_DY),
FIGURINE("PM_STORM_GIANT",       "figurine4.png", 5, 0, FIG_DX, FIG_LARGE_DY),
FIGURINE("PM_ETTIN",             "figurine4.png", 0, 1, FIG_DX, FIG_LARGE_DY),
FIGURINE("PM_TITAN",             "figurine4.png", 1, 1, FIG_DX, FIG_LARGE_DY),
FIGURINE("PM_MINOTAUR",          "figurine4.png", 2, 1, FIG_DX, FIG_LARGE_DY),
FIGURINE("PM_GIANT_ZOMBIE",      "figurine4.png", 3, 1, FIG_DX, FIG_LARGE_DY),
FIGURINE("PM_ETTIN_ZOMBIE",      "figurine4.png", 4, 1, FIG_DX, FIG_LARGE_DY),
FIGURINE("PM_GIANT_MUMMY",       "figurine4.png", 5, 1, FIG_DX, FIG_LARGE_DY),
FIGURINE("PM_ETTIN_MUMMY",       "figurine4.png", 0, 2, FIG_DX, FIG_LARGE_DY),
FIGURINE("PM_OGRE",              "figurine4.png", 1, 2, FIG_DX, FIG_LARGE_DY),
FIGURINE("PM_OGRE_LORD",         "figurine4.png", 2, 2, FIG_DX, FIG_LARGE_DY),
FIGURINE("PM_OGRE_KING",         "figurine4.png", 3, 2, FIG_DX, FIG_LARGE_DY),
FIGURINE("PM_DISENCHANTER",      "figurine4.png", 4, 2, FIG_DX, FIG_LARGE_DY),
FIGURINE("PM_SASQUATCH",         "figurine4.png", 5, 2, FIG_DX, FIG_LARGE_DY),
FIGURINE("PM_SUCCUBUS",          "figurine5.png", 0, 0, FIG_LARGE_DX, FIG_LARGE_DY),
FIGURINE("PM_INCUBUS",           "figurine5.png", 1, 0, FIG_LARGE_DX, FIG_LARGE_DY),
FIGURINE("PM_BABY_YELLOW_DRAGON","figurine5.png", 2, 0, FIG_LARGE_DX, FIG_LARGE_DY),
FIGURINE("PM_BABY_WHITE_DRAGON", "figurine5.png", 3, 0, FIG_LARGE_DX, FIG_LARGE_DY),
FIGURINE("PM_BABY_SILVER_DRAGON","figurine5.png", 4, 0, FIG_LARGE_DX, FIG_LARGE_DY),
FIGURINE("PM_BABY_RED_DRAGON",   "figurine5.png", 0, 1, FIG_LARGE_DX, FIG_LARGE_DY),
FIGURINE("PM_BABY_BLACK_DRAGON", "figurine5.png", 1, 1, FIG_LARGE_DX, FIG_LARGE_DY),
FIGURINE("PM_BABY_GRAY_DRAGON",  "figurine5.png", 2, 1, FIG_LARGE_DX, FIG_LARGE_DY),
FIGURINE("PM_BABY_GREEN_DRAGON", "figurine5.png", 3, 1, FIG_LARGE_DX, FIG_LARGE_DY),
FIGURINE("PM_BABY_ORANGE_DRAGON","figurine5.png", 4, 1, FIG_LARGE_DX, FIG_LARGE_DY),
FIGURINE("PM_BABY_BLUE_DRAGON",  "figurine5.png", 0, 2, FIG_LARGE_DX, FIG_LARGE_DY),
FIGURINE("PM_PURPLE_WORM",       "figurine5.png", 1, 2, FIG_LARGE_DX, FIG_LARGE_DY),
FIGURINE("PM_LONG_WORM_TAIL",    "figurine5.png", 2, 2, FIG_LARGE_DX, FIG_LARGE_DY),
FIGURINE("PM_LONG_WORM",         "figurine5.png", 3, 2, FIG_LARGE_DX, FIG_LARGE_DY),
FIGURINE("PM_BABY_LONG_WORM",    "figurine5.png", 4, 2, FIG_LARGE_DX, FIG_LARGE_DY),
FIGURINE("PM_SILVER_DRAGON",     "figurine6.png", 0, 0, FIG_HUGE_DX, FIG_HUGE_DY),
FIGURINE("PM_BLACK_DRAGON",      "figurine6.png", 1, 0, FIG_HUGE_DX, FIG_HUGE_DY),
FIGURINE("PM_WHITE_DRAGON",      "figurine6.png", 2, 0, FIG_HUGE_DX, FIG_HUGE_DY),
FIGURINE("PM_BALUCHITHERIUM",    "figurine6.png", 0, 1, FIG_HUGE_DX, FIG_HUGE_DY),
FIGURINE("PM_MASTODON",          "figurine6.png", 1, 1, FIG_HUGE_DX, FIG_HUGE_DY),
FIGURINE("PM_WIZARD_OF_YENDOR",  "figurine6.png", 2, 1, FIG_HUGE_DX, FIG_HUGE_DY),
FIGURINE("PM_RED_DRAGON",        "figurine6.png", 3, 0, FIG_HUGE_DX, FIG_HUGE_DY),
FIGURINE("PM_GREEN_DRAGON",      "figurine6.png", 4, 0, FIG_HUGE_DX, FIG_HUGE_DY),
FIGURINE("PM_ORANGE_DRAGON",     "figurine6.png", 5, 0, FIG_HUGE_DX, FIG_HUGE_DY),
FIGURINE("PM_YELLOW_DRAGON",     "figurine6.png", 3, 1, FIG_HUGE_DX, FIG_HUGE_DY),
FIGURINE("PM_BLUE_DRAGON",       "figurine6.png", 4, 1, FIG_HUGE_DX, FIG_HUGE_DY),
FIGURINE("PM_GRAY_DRAGON",       "figurine6.png", 5, 1, FIG_HUGE_DX, FIG_HUGE_DY),

/* statues: NAME                   FILENAME          top left x coord     top left y coord    width       height*/
/* The name field is the name of a define in pm.h */
STATUE("PM_KNIGHT",            "statue6.png", 0, 0, MON_DX, MON_DY),
STATUE("PM_GOBLIN",            "statue6.png", 1, 0, MON_DX, MON_DY),
STATUE("PM_QUEEN_BEE",         "statue6.png", 2, 0, MON_DX, MON_DY),
STATUE("PM_FIRE_ELEMENTAL",    "statue6.png", 3, 0, MON_DX, MON_DY),
STATUE("PM_GREEN_SLIME",       "statue6.png", 4, 0, MON_DX, MON_DY),
STATUE("PM_SKELETON",          "statue6.png", 5, 0, MON_DX, MON_DY),
STATUE("PM_FLOATING_EYE",      "statue6.png", 0, 1, MON_DX, MON_DY),
STATUE("PM_QUANTUM_MECHANIC",  "statue6.png", 1, 1, MON_DX, MON_DY),
STATUE("PM_HOUSECAT",          "statue6.png", 2, 1, MON_DX, MON_DY),
STATUE("PM_LIZARD",            "statue6.png", 3, 1, MON_DX, MON_DY),
STATUE("PM_LITTLE_DOG",        "statue6.png", 4, 1, MON_DX, MON_DY),
STATUE("PM_TROLL",             "statue6.png", 5, 1, MON_DX, MON_DY),
STATUE("PM_WOOD_NYMPH",        "statue6.png", 0, 2, MON_DX, MON_DY),
STATUE("PM_MOUNTAIN_NYMPH",    "statue6.png", 1, 2, MON_DX, MON_DY),
STATUE("PM_WIZARD",            "statue6.png", 2, 2, MON_DX, MON_DY),
STATUE("PM_VALKYRIE",          "statue6.png", 3, 2, MON_DX, MON_DY),
STATUE("PM_RANGER",            "statue6.png", 4, 2, MON_DX, MON_DY),
STATUE("PM_WATER_NYMPH",       "statue6.png", 5, 2, MON_DX, MON_DY),
STATUE("PM_GHOST",             "statue6.png", 0, 3, MON_DX, MON_DY),
STATUE("PM_WRAITH",            "statue6.png", 1, 3, MON_DX, MON_DY),
STATUE("PM_GIANT_BEETLE",      "statue6.png", 2, 3, MON_DX, MON_DY),
STATUE("PM_HUMAN_ZOMBIE",      "statue6.png", 3, 3, MON_DX, MON_DY),
STATUE("PM_CENTIPEDE",         "statue6.png", 4, 3, MON_DX, MON_DY),
STATUE("PM_GIANT_RAT",         "statue6.png", 5, 3, MON_DX, MON_DY),
STATUE("PM_ARCHEOLOGIST",      "statue7.png", 0, 0, MON_DX, MON_DY),
STATUE("PM_TOURIST",           "statue7.png", 1, 0, MON_DX, MON_DY),
STATUE("PM_ROGUE",             "statue7.png", 2, 0, MON_DX, MON_DY),
STATUE("PM_PRIEST",            "statue7.png", 3, 0, MON_DX, MON_DY),
STATUE("PM_COYOTE",            "statue7.png", 4, 0, MON_DX, MON_DY),
STATUE("PM_JACKAL",            "statue7.png", 5, 0, MON_DX, MON_DY),
STATUE("PM_VAMPIRE_BAT",       "statue7.png", 0, 1, MON_DX, MON_DY),
STATUE("PM_FOREST_CENTAUR",    "statue7.png", 1, 1, MON_DX, MON_DY),
STATUE("PM_SOLDIER_ANT",       "statue7.png", 2, 1, MON_DX, MON_DY),
STATUE("PM_KILLER_BEE",        "statue7.png", 3, 1, MON_DX, MON_DY),
STATUE("PM_YELLOW_LIGHT",      "statue7.png", 4, 1, MON_DX, MON_DY),
STATUE("PM_FOX",               "statue7.png", 5, 1, MON_DX, MON_DY),
STATUE("PM_COCKATRICE",        "statue7.png", 0, 2, MON_DX, MON_DY),
STATUE("PM_KRAKEN",            "statue7.png", 1, 2, MON_DX, MON_DY),
STATUE("PM_GIANT_EEL",         "statue7.png", 2, 2, MON_DX, MON_DY),
STATUE("PM_PYROLISK",          "statue7.png", 3, 2, MON_DX, MON_DY),
STATUE("PM_YELLOW_MOLD",       "statue7.png", 4, 2, MON_DX, MON_DY),
STATUE("PM_SHARK",             "statue7.png", 5, 2, MON_DX, MON_DY),
STATUE("PM_CHICKATRICE",       "statue7.png", 0, 3, MON_DX, MON_DY),
STATUE("PM_SMALL_MIMIC",       "statue7.png", 1, 3, MON_DX, MON_DY),
STATUE("PM_LARGE_MIMIC",       "statue7.png", 2, 3, MON_DX, MON_DY),
STATUE("PM_GIANT_MIMIC",       "statue7.png", 3, 3, MON_DX, MON_DY),
STATUE("PM_RED_MOLD",          "statue7.png", 4, 3, MON_DX, MON_DY),
STATUE("PM_VIOLET_FUNGUS",     "statue7.png", 5, 3, MON_DX, MON_DY),
STATUE("PM_ACID_BLOB",         "statue8.png", 0, 0, MON_DX, MON_DY),
STATUE("PM_QUIVERING_BLOB",    "statue8.png", 1, 0, MON_DX, MON_DY),
STATUE("PM_GELATINOUS_CUBE",   "statue8.png", 2, 0, MON_DX, MON_DY),
STATUE("PM_BROWN_MOLD",        "statue8.png", 3, 0, MON_DX, MON_DY),
STATUE("PM_LICHEN",            "statue8.png", 4, 0, MON_DX, MON_DY),
STATUE("PM_GREEN_MOLD",        "statue8.png", 5, 0, MON_DX, MON_DY),
STATUE("PM_SHRIEKER",          "statue8.png", 0, 1, MON_DX, MON_DY),
STATUE("PM_STALKER",           "statue8.png", 1, 1, MON_DX, MON_DY),
STATUE("PM_AIR_ELEMENTAL",     "statue8.png", 2, 1, MON_DX, MON_DY),
STATUE("PM_EARTH_ELEMENTAL",   "statue8.png", 3, 1, MON_DX, MON_DY),
STATUE("PM_WATER_ELEMENTAL",   "statue8.png", 4, 1, MON_DX, MON_DY),
STATUE("PM_BLACK_LIGHT",       "statue8.png", 5, 1, MON_DX, MON_DY),
STATUE("PM_ROCK_PIERCER",      "statue8.png", 0, 2, MON_DX, MON_DY),
STATUE("PM_IRON_PIERCER",      "statue8.png", 1, 2, MON_DX, MON_DY),
STATUE("PM_GLASS_PIERCER",     "statue8.png", 2, 2, MON_DX, MON_DY),
STATUE("PM_DOG",               "statue8.png", 3, 2, MON_DX, MON_DY),
STATUE("PM_LARGE_DOG",         "statue8.png", 4, 2, MON_DX, MON_DY),
STATUE("PM_WATER_DEMON",       "statue8.png", 5, 2, MON_DX, MON_DY),
STATUE("PM_GECKO",             "statue8.png", 0, 3, MON_DX, MON_DY),
STATUE("PM_NEWT",              "statue8.png", 1, 3, MON_DX, MON_DY),
STATUE("PM_GAS_SPORE",         "statue8.png", 3, 3, MON_DX, MON_DY),
STATUE("PM_KITTEN",            "statue8.png", 4, 3, MON_DX, MON_DY),
STATUE("PM_LARGE_CAT",         "statue8.png", 5, 3, MON_DX, MON_DY),
STATUE("PM_FREEZING_SPHERE",   "statue9.png", 0, 0, MON_DX, MON_DY),
STATUE("PM_FLAMING_SPHERE",    "statue9.png", 1, 0, MON_DX, MON_DY),
STATUE("PM_SHOCKING_SPHERE",   "statue9.png", 2, 0, MON_DX, MON_DY),
STATUE("PM_PONY",              "statue9.png", 3, 0, MON_DX, MON_DY),
STATUE("PM_HORSE",             "statue9.png", 4, 0, MON_DX, MON_DY),
STATUE("PM_WARHORSE",          "statue9.png", 5, 0, MON_DX, MON_DY),
STATUE("PM_ROTHE",             "statue9.png", 0, 1, MON_DX, MON_DY),
STATUE("PM_GNOME",             "statue9.png", 1, 1, MON_DX, MON_DY),
STATUE("PM_GREEN_ELF",         "statue9.png", 2, 1, MON_DX, MON_DY),
STATUE("PM_DWARF",             "statue9.png", 3, 1, MON_DX, MON_DY),
STATUE("PM_WATER_MOCCASIN",    "statue9.png", 4, 1, MON_DX, MON_DY),
STATUE("PM_MANES",             "statue9.png", 5, 1, MON_DX, MON_DY),
STATUE("PM_HOBBIT",            "statue9.png", 0, 2, MON_DX, MON_DY),
STATUE("PM_BLACK_UNICORN",     "statue9.png", 1, 2, MON_DX, MON_DY),
STATUE("PM_GRAY_UNICORN",      "statue9.png", 2, 2, MON_DX, MON_DY),
STATUE("PM_WHITE_UNICORN",     "statue9.png", 3, 2, MON_DX, MON_DY),
STATUE("PM_GIANT_ANT",         "statue9.png", 4, 2, MON_DX, MON_DY),
STATUE("PM_FIRE_ANT",          "statue9.png", 5, 2, MON_DX, MON_DY),
STATUE("PM_ELVENKING",         "statue9.png", 0, 3, MON_DX, MON_DY),
STATUE("PM_ELF_LORD",          "statue9.png", 1, 3, MON_DX, MON_DY),
STATUE("PM_GREY_ELF",          "statue9.png", 2, 3, MON_DX, MON_DY),
STATUE("PM_WOODLAND_ELF",      "statue9.png", 3, 3, MON_DX, MON_DY),
STATUE("PM_ELF",               "statue9.png", 4, 3, MON_DX, MON_DY),
STATUE("PM_ROCK_MOLE",         "statue9.png", 5, 3, MON_DX, MON_DY),
STATUE("PM_DWARF_LORD",        "statue0.png", 0, 0, MON_DX, MON_DY),
STATUE("PM_DWARF_KING",        "statue0.png", 1, 0, MON_DX, MON_DY),
STATUE("PM_GNOME_LORD",        "statue0.png", 2, 0, MON_DX, MON_DY),
STATUE("PM_GNOME_KING",        "statue0.png", 3, 0, MON_DX, MON_DY),
STATUE("PM_GNOMISH_WIZARD",    "statue0.png", 4, 0, MON_DX, MON_DY),
STATUE("PM_ORC",               "statue0.png", 5, 0, MON_DX, MON_DY),
STATUE("PM_HILL_ORC",          "statue0.png", 0, 1, MON_DX, MON_DY),
STATUE("PM_MORDOR_ORC",        "statue0.png", 1, 1, MON_DX, MON_DY),
STATUE("PM_URUK_HAI",          "statue0.png", 2, 1, MON_DX, MON_DY),
STATUE("PM_HOBGOBLIN",         "statue0.png", 3, 1, MON_DX, MON_DY),
STATUE("PM_BUGBEAR",           "statue0.png", 4, 1, MON_DX, MON_DY),
STATUE("PM_KOBOLD",            "statue0.png", 5, 1, MON_DX, MON_DY),
STATUE("PM_LARGE_KOBOLD",      "statue0.png", 0, 2, MON_DX, MON_DY),
STATUE("PM_KOBOLD_SHAMAN",     "statue0.png", 1, 2, MON_DX, MON_DY),
STATUE("PM_KOBOLD_LORD",       "statue0.png", 2, 2, MON_DX, MON_DY),
STATUE("PM_ORC_CAPTAIN",       "statue0.png", 3, 2, MON_DX, MON_DY),
STATUE("PM_ORC_SHAMAN",        "statue0.png", 4, 2, MON_DX, MON_DY),
STATUE("PM_LEPRECHAUN",        "statue0.png", 5, 2, MON_DX, MON_DY),
STATUE("PM_GARTER_SNAKE",      "statue0.png", 0, 3, MON_DX, MON_DY),
STATUE("PM_SNAKE",             "statue0.png", 1, 3, MON_DX, MON_DY),
STATUE("PM_PIT_VIPER",         "statue0.png", 2, 3, MON_DX, MON_DY),
STATUE("PM_PYTHON",            "statue0.png", 3, 3, MON_DX, MON_DY),
STATUE("PM_COBRA",             "statue0.png", 4, 3, MON_DX, MON_DY),
STATUE("PM_WUMPUS",            "statue0.png", 5, 3, MON_DX, MON_DY),
STATUE("PM_KOBOLD_ZOMBIE",     "statue1.png", 0, 0, MON_DX, MON_DY),
STATUE("PM_GNOME_ZOMBIE",      "statue1.png", 1, 0, MON_DX, MON_DY),
STATUE("PM_DWARF_ZOMBIE",      "statue1.png", 2, 0, MON_DX, MON_DY),
STATUE("PM_ORC_ZOMBIE",        "statue1.png", 3, 0, MON_DX, MON_DY),
STATUE("PM_ELF_ZOMBIE",        "statue1.png", 4, 0, MON_DX, MON_DY),
STATUE("PM_KOBOLD_MUMMY",      "statue1.png", 5, 0, MON_DX, MON_DY),
STATUE("PM_GNOME_MUMMY",       "statue1.png", 0, 1, MON_DX, MON_DY),
STATUE("PM_DWARF_MUMMY",       "statue1.png", 1, 1, MON_DX, MON_DY),
STATUE("PM_ORC_MUMMY",         "statue1.png", 2, 1, MON_DX, MON_DY),
STATUE("PM_HUMAN_MUMMY",       "statue1.png", 3, 1, MON_DX, MON_DY),
STATUE("PM_ELF_MUMMY",         "statue1.png", 4, 1, MON_DX, MON_DY),
STATUE("PM_HOMUNCULUS",        "statue1.png", 5, 1, MON_DX, MON_DY),
STATUE("PM_IMP",               "statue1.png", 0, 2, MON_DX, MON_DY),
STATUE("PM_LEMURE",            "statue1.png", 1, 2, MON_DX, MON_DY),
STATUE("PM_QUASIT",            "statue1.png", 2, 2, MON_DX, MON_DY),
STATUE("PM_TENGU",             "statue1.png", 3, 2, MON_DX, MON_DY),
STATUE("PM_ORACLE",            "statue1.png", 4, 2, MON_DX, MON_DY),
STATUE("PM_ALIGNED_PRIEST",    "statue1.png", 5, 2, MON_DX, MON_DY),
STATUE("PM_WATCHMAN",          "statue1.png", 3, 3, MON_DX, MON_DY),
STATUE("PM_WATCH_CAPTAIN",     "statue1.png", 4, 3, MON_DX, MON_DY),
STATUE("PM_SOLDIER",           "statue1.png", 5, 3, MON_DX, MON_DY),
STATUE("PM_SERGEANT",          "statue2.png", 0, 0, MON_DX, MON_DY),
STATUE("PM_LIEUTENANT",        "statue2.png", 1, 0, MON_DX, MON_DY),
STATUE("PM_CAPTAIN",           "statue2.png", 2, 0, MON_DX, MON_DY),
STATUE("PM_GRID_BUG",          "statue2.png", 3, 0, MON_DX, MON_DY),
STATUE("PM_XAN",               "statue2.png", 4, 0, MON_DX, MON_DY),
STATUE("PM_GARGOYLE",          "statue2.png", 5, 0, MON_DX, MON_DY),
STATUE("PM_WINGED_GARGOYLE",   "statue2.png", 0, 1, MON_DX, MON_DY),
STATUE("PM_DINGO",             "statue2.png", 1, 1, MON_DX, MON_DY),
STATUE("PM_WOLF",              "statue2.png", 2, 1, MON_DX, MON_DY),
STATUE("PM_WEREWOLF",          "statue2.png", 3, 1, MON_DX, MON_DY),
STATUE("PM_WARG",              "statue2.png", 4, 1, MON_DX, MON_DY),
STATUE("PM_WINTER_WOLF_CUB",   "statue2.png", 5, 1, MON_DX, MON_DY),
STATUE("PM_WINTER_WOLF",       "statue2.png", 0, 2, MON_DX, MON_DY),
STATUE("PM_HELL_HOUND_PUP",    "statue2.png", 1, 2, MON_DX, MON_DY),
STATUE("PM_HELL_HOUND",        "statue2.png", 2, 2, MON_DX, MON_DY),
STATUE("PM_JAGUAR",            "statue2.png", 3, 2, MON_DX, MON_DY),
STATUE("PM_LYNX",              "statue2.png", 4, 2, MON_DX, MON_DY),
STATUE("PM_PANTHER",           "statue2.png", 5, 2, MON_DX, MON_DY),
STATUE("PM_TIGER",             "statue2.png", 0, 3, MON_DX, MON_DY),
STATUE("PM_MIND_FLAYER",       "statue2.png", 1, 3, MON_DX, MON_DY),
STATUE("PM_MASTER_MIND_FLAYER","statue2.png", 2, 3, MON_DX, MON_DY),
STATUE("PM_LICH",              "statue2.png", 3, 3, MON_DX, MON_DY),
STATUE("PM_DEMILICH",          "statue2.png", 4, 3, MON_DX, MON_DY),
STATUE("PM_MASTER_LICH",       "statue2.png", 5, 3, MON_DX, MON_DY),
STATUE("PM_ARCH_LICH",         "statue3.png", 0, 0, MON_DX, MON_DY),
STATUE("PM_GUARDIAN_NAGA",     "statue3.png", 1, 0, MON_DX, MON_DY),
STATUE("PM_GUARDIAN_NAGA_HATCHLING","statue3.png", 2, 0, MON_DX, MON_DY),
STATUE("PM_GOLDEN_NAGA",       "statue3.png", 3, 0, MON_DX, MON_DY),
STATUE("PM_GOLDEN_NAGA_HATCHLING","statue3.png", 4, 0, MON_DX, MON_DY),
STATUE("PM_RED_NAGA",          "statue3.png", 5, 0, MON_DX, MON_DY),
STATUE("PM_RED_NAGA_HATCHLING","statue3.png", 0, 1, MON_DX, MON_DY),
STATUE("PM_BLACK_NAGA",        "statue3.png", 1, 1, MON_DX, MON_DY),
STATUE("PM_BLACK_NAGA_HATCHLING","statue3.png", 2, 1, MON_DX, MON_DY),
STATUE("PM_RUST_MONSTER",      "statue3.png", 3, 1, MON_DX, MON_DY),
STATUE("PM_GREMLIN",           "statue3.png", 4, 1, MON_DX, MON_DY),
STATUE("PM_BABY_PURPLE_WORM",  "statue3.png", 5, 1, MON_DX, MON_DY),
STATUE("PM_MONKEY",            "statue3.png", 0, 2, MON_DX, MON_DY),
STATUE("PM_APE",               "statue3.png", 1, 2, MON_DX, MON_DY),
STATUE("PM_OWLBEAR",           "statue3.png", 2, 2, MON_DX, MON_DY),
STATUE("PM_YETI",              "statue3.png", 3, 2, MON_DX, MON_DY),
STATUE("PM_CARNIVOROUS_APE",   "statue3.png", 4, 2, MON_DX, MON_DY),
STATUE("PM_STRAW_GOLEM",       "statue3.png", 5, 2, MON_DX, MON_DY),
STATUE("PM_PAPER_GOLEM",       "statue3.png", 0, 3, MON_DX, MON_DY),
STATUE("PM_ROPE_GOLEM",        "statue3.png", 1, 3, MON_DX, MON_DY),
STATUE("PM_GOLD_GOLEM",        "statue3.png", 2, 3, MON_DX, MON_DY),
STATUE("PM_LEATHER_GOLEM",     "statue3.png", 3, 3, MON_DX, MON_DY),
STATUE("PM_WOOD_GOLEM",        "statue3.png", 4, 3, MON_DX, MON_DY),
STATUE("PM_FLESH_GOLEM",       "statue3.png", 5, 3, MON_DX, MON_DY),
STATUE("PM_GIANT",             "statue4.png", 0, 0, MON_DX, MON_LARGE_DY),
STATUE("PM_STONE_GIANT",       "statue4.png", 1, 0, MON_DX, MON_LARGE_DY),
STATUE("PM_HILL_GIANT",        "statue4.png", 2, 0, MON_DX, MON_LARGE_DY),
STATUE("PM_FROST_GIANT",       "statue4.png", 3, 0, MON_DX, MON_LARGE_DY),
STATUE("PM_FIRE_GIANT",        "statue4.png", 4, 0, MON_DX, MON_LARGE_DY),
STATUE("PM_STORM_GIANT",       "statue4.png", 5, 0, MON_DX, MON_LARGE_DY),
STATUE("PM_ETTIN",             "statue4.png", 0, 1, MON_DX, MON_LARGE_DY),
STATUE("PM_TITAN",             "statue4.png", 1, 1, MON_DX, MON_LARGE_DY),
STATUE("PM_MINOTAUR",          "statue4.png", 2, 1, MON_DX, MON_LARGE_DY),
STATUE("PM_GIANT_ZOMBIE",      "statue4.png", 3, 1, MON_DX, MON_LARGE_DY),
STATUE("PM_ETTIN_ZOMBIE",      "statue4.png", 4, 1, MON_DX, MON_LARGE_DY),
STATUE("PM_GIANT_MUMMY",       "statue4.png", 5, 1, MON_DX, MON_LARGE_DY),
STATUE("PM_ETTIN_MUMMY",       "statue4.png", 0, 2, MON_DX, MON_LARGE_DY),
STATUE("PM_OGRE",              "statue4.png", 1, 2, MON_DX, MON_LARGE_DY),
STATUE("PM_OGRE_LORD",         "statue4.png", 2, 2, MON_DX, MON_LARGE_DY),
STATUE("PM_OGRE_KING",         "statue4.png", 3, 2, MON_DX, MON_LARGE_DY),
STATUE("PM_DISENCHANTER",      "statue4.png", 4, 2, MON_DX, MON_LARGE_DY),
STATUE("PM_SASQUATCH",         "statue4.png", 5, 2, MON_DX, MON_LARGE_DY),
STATUE("PM_SUCCUBUS",          "statue5.png", 0, 0, MON_LARGE_DX, MON_LARGE_DY),
STATUE("PM_INCUBUS",           "statue5.png", 1, 0, MON_LARGE_DX, MON_LARGE_DY),
STATUE("PM_BABY_YELLOW_DRAGON","statue5.png", 2, 0, MON_LARGE_DX, MON_LARGE_DY),
STATUE("PM_BABY_WHITE_DRAGON", "statue5.png", 3, 0, MON_LARGE_DX, MON_LARGE_DY),
STATUE("PM_BABY_SILVER_DRAGON","statue5.png", 4, 0, MON_LARGE_DX, MON_LARGE_DY),
STATUE("PM_BABY_RED_DRAGON",   "statue5.png", 0, 1, MON_LARGE_DX, MON_LARGE_DY),
STATUE("PM_BABY_BLACK_DRAGON", "statue5.png", 1, 1, MON_LARGE_DX, MON_LARGE_DY),
STATUE("PM_BABY_GRAY_DRAGON",  "statue5.png", 2, 1, MON_LARGE_DX, MON_LARGE_DY),
STATUE("PM_BABY_GREEN_DRAGON", "statue5.png", 3, 1, MON_LARGE_DX, MON_LARGE_DY),
STATUE("PM_BABY_ORANGE_DRAGON","statue5.png", 4, 1, MON_LARGE_DX, MON_LARGE_DY),
STATUE("PM_BABY_BLUE_DRAGON",  "statue5.png", 0, 2, MON_LARGE_DX, MON_LARGE_DY),
STATUE("PM_PURPLE_WORM",       "statue5.png", 1, 2, MON_LARGE_DX, MON_LARGE_DY),
STATUE("PM_LONG_WORM_TAIL",    "statue5.png", 2, 2, MON_LARGE_DX, MON_LARGE_DY),
STATUE("PM_LONG_WORM",         "statue5.png", 3, 2, MON_LARGE_DX, MON_LARGE_DY),
STATUE("PM_BABY_LONG_WORM",    "statue5.png", 4, 2, MON_LARGE_DX, MON_LARGE_DY),
STATUE("PM_SCORPION",          "statue10.png", 0, 0, MON_DX, MON_DY),
STATUE("PM_CAVE_SPIDER",       "statue10.png", 1, 0, MON_DX, MON_DY),
STATUE("PM_GIANT_SPIDER",      "statue10.png", 2, 0, MON_DX, MON_DY),
STATUE("PM_BAT",               "statue10.png", 3, 0, MON_DX, MON_DY),
STATUE("PM_GIANT_BAT",         "statue10.png", 4, 0, MON_DX, MON_DY),
STATUE("PM_RAVEN",             "statue10.png", 5, 0, MON_DX, MON_DY),
STATUE("PM_DJINNI",            "statue10.png", 0, 1, MON_DX, MON_DY),
STATUE("PM_MOUNTAIN_CENTAUR",  "statue10.png", 1, 1, MON_DX, MON_DY),
STATUE("PM_PLAINS_CENTAUR",    "statue10.png", 2, 1, MON_DX, MON_DY),
STATUE("PM_COUATL",            "statue10.png", 3, 1, MON_DX, MON_DY),
STATUE("PM_DOPPELGANGER",      "statue10.png", 4, 1, MON_DX, MON_DY),
STATUE("PM_CROESUS",           "statue10.png", 5, 1, MON_DX, MON_DY),
STATUE("PM_KEYSTONE_KOP",      "statue10.png", 0, 2, MON_DX, MON_DY),
STATUE("PM_KOP_SERGEANT",      "statue10.png", 1, 2, MON_DX, MON_DY),
STATUE("PM_KOP_LIEUTENANT",    "statue10.png", 2, 2, MON_DX, MON_DY),
STATUE("PM_KOP_KAPTAIN",       "statue10.png", 3, 2, MON_DX, MON_DY),
STATUE("PM_VAMPIRE",           "statue10.png", 4, 2, MON_DX, MON_DY),
STATUE("PM_SEWER_RAT",         "statue10.png", 5, 2, MON_DX, MON_DY),
STATUE("PM_RABID_RAT",         "statue10.png", 0, 3, MON_DX, MON_DY),
STATUE("PM_WERERAT",           "statue10.png", 1, 3, MON_DX, MON_DY),
STATUE("PM_WOODCHUCK",         "statue10.png", 2, 3, MON_DX, MON_DY),
STATUE("PM_MEDUSA",            "statue10.png", 3, 3, MON_DX, MON_DY),
STATUE("PM_HIGH_PRIEST",       "statue10.png", 4, 3, MON_DX, MON_DY),
STATUE("PM_NURSE",             "statue10.png", 5, 3, MON_DX, MON_DY),
STATUE("PM_SILVER_DRAGON",     "statue14.png", 0, 0, MON_HUGE_DX, MON_HUGE_DY),
STATUE("PM_BLACK_DRAGON",      "statue14.png", 1, 0, MON_HUGE_DX, MON_HUGE_DY),
STATUE("PM_WHITE_DRAGON",      "statue14.png", 2, 0, MON_HUGE_DX, MON_HUGE_DY),
STATUE("PM_BALUCHITHERIUM",    "statue14.png", 0, 1, MON_HUGE_DX, MON_HUGE_DY),
STATUE("PM_MASTODON",          "statue14.png", 1, 1, MON_HUGE_DX, MON_HUGE_DY),
STATUE("PM_WIZARD_OF_YENDOR",  "statue14.png", 2, 1, MON_HUGE_DX, MON_HUGE_DY),
STATUE("PM_RED_DRAGON",        "statue15.png", 0, 0, MON_HUGE_DX, MON_HUGE_DY),
STATUE("PM_GREEN_DRAGON",      "statue15.png", 1, 0, MON_HUGE_DX, MON_HUGE_DY),
STATUE("PM_ORANGE_DRAGON",     "statue15.png", 2, 0, MON_HUGE_DX, MON_HUGE_DY),
STATUE("PM_YELLOW_DRAGON",     "statue15.png", 0, 1, MON_HUGE_DX, MON_HUGE_DY),
STATUE("PM_BLUE_DRAGON",       "statue15.png", 1, 1, MON_HUGE_DX, MON_HUGE_DY),
STATUE("PM_GRAY_DRAGON",       "statue15.png", 2, 1, MON_HUGE_DX, MON_HUGE_DY),

/* Absurd tileset mappings */
/* statues: NAME                    FILENAME                 top left x coord     top left y coord    width       height*/
// MONSTER("PM_GIANT_ANT",          "monster-absurd1.png", 0, 0, MON_DX, MON_DY),
// MONSTER("PM_KILLER_BEE",         "monster-absurd1.png", 1, 0, MON_DX, MON_DY),
// MONSTER("PM_SOLDIER_ANT",        "monster-absurd1.png", 2, 0, MON_DX, MON_DY),
// MONSTER("PM_FIRE_ANT",           "monster-absurd1.png", 3, 0, MON_DX, MON_DY),
// MONSTER("PM_GIANT_BEETLE",       "monster-absurd1.png", 4, 0, MON_DX, MON_DY),
// MONSTER("PM_QUEEN_BEE",          "monster-absurd1.png", 5, 0, MON_DX, MON_DY),
// MONSTER("PM_ACID_BLOB",          "monster-absurd1.png", 0, 1, MON_DX, MON_DY),
// MONSTER("PM_QUIVERING_BLOB",     "monster-absurd1.png", 1, 1, MON_DX, MON_DY),
// MONSTER("PM_GELATINOUS_CUBE",    "monster-absurd1.png", 2, 1, MON_DX, MON_DY),
// MONSTER("PM_CHICKATRICE",        "monster-absurd1.png", 3, 1, MON_DX, MON_DY),
// MONSTER("PM_COCKATRICE",         "monster-absurd1.png", 4, 1, MON_DX, MON_DY),
// MONSTER("PM_PYROLISK",           "monster-absurd1.png", 5, 1, MON_DX, MON_DY),
// MONSTER("PM_JACKAL",             "monster-absurd1.png", 0, 2, MON_DX, MON_DY),
// MONSTER("PM_FOX",                "monster-absurd1.png", 1, 2, MON_DX, MON_DY),
// MONSTER("PM_COYOTE",             "monster-absurd1.png", 2, 2, MON_DX, MON_DY),
MONSTER("PM_WEREJACKAL",         "monster-absurd1.png", 3, 2, MON_DX, MON_DY),
// MONSTER("PM_LITTLE_DOG",         "monster-absurd1.png", 4, 2, MON_DX, MON_DY),
// MONSTER("PM_DOG",                "monster-absurd1.png", 5, 2, MON_DX, MON_DY),
// MONSTER("PM_LARGE_DOG",          "monster-absurd1.png", 0, 3, MON_DX, MON_DY),
// MONSTER("PM_DINGO",              "monster-absurd1.png", 1, 3, MON_DX, MON_DY),
// MONSTER("PM_WOLF",               "monster-absurd1.png", 2, 3, MON_DX, MON_DY),
// MONSTER("PM_WEREWOLF",           "monster-absurd1.png", 3, 3, MON_DX, MON_DY),
// MONSTER("PM_WARG",               "monster-absurd1.png", 4, 3, MON_DX, MON_DY),
// MONSTER("PM_WINTER_WOLF_CUB",    "monster-absurd1.png", 5, 3, MON_DX, MON_DY),

// MONSTER("PM_WINTER_WOLF",        "monster-absurd2.png", 0, 0, MON_DX, MON_DY),
// MONSTER("PM_HELL_HOUND_PUP",     "monster-absurd2.png", 1, 0, MON_DX, MON_DY),
// MONSTER("PM_HELL_HOUND",         "monster-absurd2.png", 2, 0, MON_DX, MON_DY),
MONSTER("PM_CERBERUS",           "monster-absurd2.png", 3, 0, MON_DX, MON_DY),
// MONSTER("PM_GAS_SPORE",          "monster-absurd2.png", 4, 0, MON_DX, MON_DY),
// MONSTER("PM_FLOATING_EYE",       "monster-absurd2.png", 5, 0, MON_DX, MON_DY),
// MONSTER("PM_FREEZING_SPHERE",    "monster-absurd2.png", 0, 1, MON_DX, MON_DY),
// MONSTER("PM_FLAMING_SPHERE",     "monster-absurd2.png", 1, 1, MON_DX, MON_DY),
// MONSTER("PM_SHOCKING_SPHERE",    "monster-absurd2.png", 2, 1, MON_DX, MON_DY),
MONSTER("PM_BEHOLDER",           "monster-absurd2.png", 3, 1, MON_DX, MON_DY),
// MONSTER("PM_KITTEN",             "monster-absurd2.png", 4, 1, MON_DX, MON_DY),
// MONSTER("PM_HOUSECAT",           "monster-absurd2.png", 5, 1, MON_DX, MON_DY),
// MONSTER("PM_JAGUAR",             "monster-absurd2.png", 0, 2, MON_DX, MON_DY),
// MONSTER("PM_LYNX",               "monster-absurd2.png", 1, 2, MON_DX, MON_DY),
// MONSTER("PM_PANTHER",            "monster-absurd2.png", 2, 2, MON_DX, MON_DY),
// MONSTER("PM_LARGE_CAT",          "monster-absurd2.png", 3, 2, MON_DX, MON_DY),
// MONSTER("PM_TIGER",              "monster-absurd2.png", 4, 2, MON_DX, MON_DY),
// MONSTER("PM_GREMLIN",            "monster-absurd2.png", 5, 2, MON_DX, MON_DY),
// MONSTER("PM_GARGOYLE",           "monster-absurd2.png", 0, 3, MON_DX, MON_DY),
// MONSTER("PM_WINGED_GARGOYLE",    "monster-absurd2.png", 1, 3, MON_DX, MON_DY),
// MONSTER("PM_HOBBIT",             "monster-absurd2.png", 2, 3, MON_DX, MON_DY),
// MONSTER("PM_DWARF",              "monster-absurd2.png", 3, 3, MON_DX, MON_DY),
// MONSTER("PM_BUGBEAR",            "monster-absurd2.png", 4, 3, MON_DX, MON_DY),
// MONSTER("PM_DWARF_LORD",         "monster-absurd2.png", 5, 3, MON_DX, MON_DY),

// MONSTER("PM_DWARF_KING",         "monster-absurd3.png", 0, 0, MON_DX, MON_DY),
// MONSTER("PM_MIND_FLAYER",        "monster-absurd3.png", 1, 0, MON_DX, MON_DY),
// MONSTER("PM_MASTER_MIND_FLAYER", "monster-absurd3.png", 2, 0, MON_DX, MON_DY),
MONSTER("PM_MANES",              "monster-absurd3.png", 3, 0, MON_DX, MON_DY),
// MONSTER("PM_HOMUNCULUS",         "monster-absurd3.png", 4, 0, MON_DX, MON_DY),
// MONSTER("PM_IMP",                "monster-absurd3.png", 5, 0, MON_DX, MON_DY),
MONSTER("PM_LEMURE",             "monster-absurd3.png", 0, 1, MON_DX, MON_DY),
// MONSTER("PM_QUASIT",             "monster-absurd3.png", 1, 1, MON_DX, MON_DY),
// MONSTER("PM_TENGU",              "monster-absurd3.png", 2, 1, MON_DX, MON_DY),
// MONSTER("PM_BLUE_JELLY",         "monster-absurd3.png", 3, 1, MON_DX, MON_DY),
// MONSTER("PM_SPOTTED_JELLY",      "monster-absurd3.png", 4, 1, MON_DX, MON_DY),
// MONSTER("PM_OCHRE_JELLY",        "monster-absurd3.png", 5, 1, MON_DX, MON_DY),
MONSTER("PM_KOBOLD",             "monster-absurd3.png", 0, 2, MON_DX, MON_DY),
MONSTER("PM_LARGE_KOBOLD",       "monster-absurd3.png", 1, 2, MON_DX, MON_DY),
MONSTER("PM_KOBOLD_LORD",        "monster-absurd3.png", 2, 2, MON_DX, MON_DY),
MONSTER("PM_KOBOLD_SHAMAN",      "monster-absurd3.png", 3, 2, MON_DX, MON_DY),
// MONSTER("PM_LEPRECHAUN",         "monster-absurd3.png", 4, 2, MON_DX, MON_DY),
// MONSTER("PM_SMALL_MIMIC",        "monster-absurd3.png", 5, 2, MON_DX, MON_DY),
// MONSTER("PM_LARGE_MIMIC",        "monster-absurd3.png", 0, 3, MON_DX, MON_DY),
// MONSTER("PM_GIANT_MIMIC",        "monster-absurd3.png", 1, 3, MON_DX, MON_DY),
// MONSTER("PM_WOOD_NYMPH",         "monster-absurd3.png", 2, 3, MON_DX, MON_DY),
// MONSTER("PM_WATER_NYMPH",        "monster-absurd3.png", 3, 3, MON_DX, MON_DY),
// MONSTER("PM_MOUNTAIN_NYMPH",     "monster-absurd3.png", 4, 3, MON_DX, MON_DY),
MONSTER("PM_GOBLIN",             "monster-absurd3.png", 5, 3, MON_DX, MON_DY),

MONSTER("PM_HOBGOBLIN",          "monster-absurd4.png", 0, 0, MON_DX, MON_DY),
MONSTER("PM_ORC",                "monster-absurd4.png", 1, 0, MON_DX, MON_DY),
MONSTER("PM_HILL_ORC",           "monster-absurd4.png", 2, 0, MON_DX, MON_DY),
MONSTER("PM_MORDOR_ORC",         "monster-absurd4.png", 3, 0, MON_DX, MON_DY),
MONSTER("PM_URUK_HAI",           "monster-absurd4.png", 4, 0, MON_DX, MON_DY),
MONSTER("PM_ORC_SHAMAN",         "monster-absurd4.png", 5, 0, MON_DX, MON_DY),
MONSTER("PM_ORC_CAPTAIN",        "monster-absurd4.png", 0, 1, MON_DX, MON_DY),
// MONSTER("PM_ROCK_PIERCER",       "monster-absurd4.png", 1, 1, MON_DX, MON_DY),
// MONSTER("PM_IRON_PIERCER",       "monster-absurd4.png", 2, 1, MON_DX, MON_DY),
// MONSTER("PM_GLASS_PIERCER",      "monster-absurd4.png", 3, 1, MON_DX, MON_DY),
// MONSTER("PM_ROTHE",              "monster-absurd4.png", 4, 1, MON_DX, MON_DY),
// MONSTER("PM_MUMAK",              "monster-absurd4.png", 5, 1, MON_DX, MON_DY),
// MONSTER("PM_LEOCROTTA",          "monster-absurd4.png", 0, 2, MON_DX, MON_DY),
// MONSTER("PM_WUMPUS",             "monster-absurd4.png", 1, 2, MON_DX, MON_DY),
// MONSTER("PM_TITANOTHERE",        "monster-absurd4.png", 2, 2, MON_DX, MON_DY),
// MONSTER("PM_BALUCHITHERIUM",     "monster-absurd4.png", 3, 2, MON_DX, MON_DY),
// MONSTER("PM_MASTODON",           "monster-absurd4.png", 4, 2, MON_DX, MON_DY),
// MONSTER("PM_SEWER_RAT",          "monster-absurd4.png", 5, 2, MON_DX, MON_DY),
// MONSTER("PM_GIANT_RAT",          "monster-absurd4.png", 0, 3, MON_DX, MON_DY),
// MONSTER("PM_RABID_RAT",          "monster-absurd4.png", 1, 3, MON_DX, MON_DY),
// MONSTER("PM_WERERAT",            "monster-absurd4.png", 2, 3, MON_DX, MON_DY),
// MONSTER("PM_ROCK_MOLE",          "monster-absurd4.png", 3, 3, MON_DX, MON_DY),
// MONSTER("PM_WOODCHUCK",          "monster-absurd4.png", 4, 3, MON_DX, MON_DY),
// MONSTER("PM_CAVE_SPIDER",        "monster-absurd4.png", 5, 3, MON_DX, MON_DY),

// MONSTER("PM_CENTIPEDE",          "monster-absurd5.png", 0, 0, MON_DX, MON_DY),
// MONSTER("PM_GIANT_SPIDER",       "monster-absurd5.png", 1, 0, MON_DX, MON_DY),
// MONSTER("PM_SCORPION",           "monster-absurd5.png", 2, 0, MON_DX, MON_DY),
// MONSTER("PM_LURKER_ABOVE",       "monster-absurd5.png", 3, 0, MON_DX, MON_DY),
// MONSTER("PM_TRAPPER",            "monster-absurd5.png", 4, 0, MON_DX, MON_DY),
// MONSTER("PM_WHITE_UNICORN",      "monster-absurd5.png", 5, 0, MON_DX, MON_DY),
// MONSTER("PM_GRAY_UNICORN",       "monster-absurd5.png", 0, 1, MON_DX, MON_DY),
// MONSTER("PM_BLACK_UNICORN",      "monster-absurd5.png", 1, 1, MON_DX, MON_DY),
// MONSTER("PM_PONY",               "monster-absurd5.png", 2, 1, MON_DX, MON_DY),
// MONSTER("PM_HORSE",              "monster-absurd5.png", 3, 1, MON_DX, MON_DY),
// MONSTER("PM_WARHORSE",           "monster-absurd5.png", 4, 1, MON_DX, MON_DY),
MONSTER("PM_FOG_CLOUD",          "monster-absurd5.png", 5, 1, MON_DX, MON_DY),
MONSTER("PM_DUST_VORTEX",        "monster-absurd5.png", 0, 2, MON_DX, MON_DY),
MONSTER("PM_ICE_VORTEX",         "monster-absurd5.png", 1, 2, MON_DX, MON_DY),
MONSTER("PM_ENERGY_VORTEX",      "monster-absurd5.png", 2, 2, MON_DX, MON_DY),
MONSTER("PM_STEAM_VORTEX",       "monster-absurd5.png", 3, 2, MON_DX, MON_DY),
MONSTER("PM_FIRE_VORTEX",        "monster-absurd5.png", 4, 2, MON_DX, MON_DY),
// MONSTER("PM_BABY_LONG_WORM",     "monster-absurd5.png", 5, 2, MON_DX, MON_DY),
// MONSTER("PM_BABY_PURPLE_WORM",   "monster-absurd5.png", 0, 3, MON_DX, MON_DY),
// MONSTER("PM_LONG_WORM",          "monster-absurd5.png", 1, 3, MON_DX, MON_DY),
// MONSTER("PM_PURPLE_WORM",        "monster-absurd5.png", 2, 3, MON_DX, MON_DY),
// MONSTER("PM_GRID_BUG",           "monster-absurd5.png", 3, 3, MON_DX, MON_DY),
// MONSTER("PM_XAN",                "monster-absurd5.png", 4, 3, MON_DX, MON_DY),
// MONSTER("PM_YELLOW_LIGHT",       "monster-absurd5.png", 5, 3, MON_DX, MON_DY),

// MONSTER("PM_BLACK_LIGHT",        "monster-absurd6.png", 0, 0, MON_DX, MON_DY),
// MONSTER("PM_ZRUTY",              "monster-absurd6.png", 1, 0, MON_DX, MON_DY),
// MONSTER("PM_COUATL",             "monster-absurd6.png", 2, 0, MON_DX, MON_DY),
// MONSTER("PM_ALEAX",              "monster-absurd6.png", 3, 0, MON_DX, MON_DY),
// MONSTER("PM_ANGEL",              "monster-absurd6.png", 4, 0, MON_DX, MON_DY),
// MONSTER("PM_KI_RIN",             "monster-absurd6.png", 5, 0, MON_DX, MON_DY),
// MONSTER("PM_ARCHON",             "monster-absurd6.png", 0, 1, MON_DX, MON_DY),
// MONSTER("PM_BAT",                "monster-absurd6.png", 1, 1, MON_DX, MON_DY),
// MONSTER("PM_GIANT_BAT",          "monster-absurd6.png", 2, 1, MON_DX, MON_DY),
// MONSTER("PM_RAVEN",              "monster-absurd6.png", 3, 1, MON_DX, MON_DY),
// MONSTER("PM_VAMPIRE_BAT",        "monster-absurd6.png", 4, 1, MON_DX, MON_DY),
// MONSTER("PM_PLAINS_CENTAUR",     "monster-absurd6.png", 5, 1, MON_DX, MON_DY),
// MONSTER("PM_FOREST_CENTAUR",     "monster-absurd6.png", 0, 2, MON_DX, MON_DY),
// MONSTER("PM_BABY_GRAY_DRAGON",   "monster-absurd6.png", 1, 2, MON_DX, MON_DY),
// MONSTER("PM_BABY_SILVER_DRAGON", "monster-absurd6.png", 2, 2, MON_DX, MON_DY),
MONSTER("PM_BABY_SHIMMERING_DRAGON", "monster-absurd6.png", 3, 2, MON_DX, MON_DY),
// MONSTER("PM_BABY_RED_DRAGON",    "monster-absurd6.png", 4, 2, MON_DX, MON_DY),
// MONSTER("PM_BABY_WHITE_DRAGON",  "monster-absurd6.png", 5, 2, MON_DX, MON_DY),
// MONSTER("PM_BABY_ORANGE_DRAGON", "monster-absurd6.png", 0, 3, MON_DX, MON_DY),
// MONSTER("PM_BABY_BLACK_DRAGON",  "monster-absurd6.png", 1, 3, MON_DX, MON_DY),
// MONSTER("PM_BABY_BLUE_DRAGON",   "monster-absurd6.png", 2, 3, MON_DX, MON_DY),
// MONSTER("PM_BABY_GREEN_DRAGON",  "monster-absurd6.png", 3, 3, MON_DX, MON_DY),
// MONSTER("PM_BABY_YELLOW_DRAGON", "monster-absurd6.png", 4, 3, MON_DX, MON_DY),
// MONSTER("PM_GRAY_DRAGON",        "monster-absurd6.png", 5, 3, MON_DX, MON_DY),

/* Second set of Absurd tileset mappings */
/* statues: NAME                    FILENAME           x coord    y coord    width       height*/
// MONSTER("PM_SOLDIER",            "monster-absurd7.png", 0, 0, MON_DX, MON_DY),
// MONSTER("PM_SERGEANT",           "monster-absurd7.png", 1, 0, MON_DX, MON_DY),
// MONSTER("PM_LIEUTENANT",         "monster-absurd7.png", 2, 0, MON_DX, MON_DY),
// MONSTER("PM_CAPTAIN",            "monster-absurd7.png", 3, 0, MON_DX, MON_DY),
// MONSTER("PM_WATCHMAN",           "monster-absurd7.png", 4, 0, MON_DX, MON_DY),
// MONSTER("PM_WATCH_CAPTAIN",      "monster-absurd7.png", 5, 0, MON_DX, MON_DY),
MONSTER("PM_ALEAX",              "monster-absurd7.png", 0, 1, MON_DX, MON_DY),
MONSTER("PM_ARCH_PRIEST",        "monster-absurd7.png", 1, 1, MON_DX, MON_DY),
MONSTER("PM_ASHIKAGA_TAKAUJI",   "monster-absurd7.png", 2, 1, MON_DX, MON_DY),
MONSTER("PM_ASMODEUS",           "monster-absurd7.png", 3, 1, MON_DX, MON_DY),
MONSTER("PM_BAALZEBUB",          "monster-absurd7.png", 4, 1, MON_DX, MON_DY),
MONSTER("PM_CHARON",             "monster-absurd7.png", 5, 1, MON_DX, MON_DY),
MONSTER("PM_CHROMATIC_DRAGON",   "monster-absurd7.png", 0, 2, MON_DX, MON_DY),
MONSTER("PM_CYCLOPS",            "monster-absurd7.png", 1, 2, MON_DX, MON_DY),
MONSTER("PM_DARK_ONE",           "monster-absurd7.png", 2, 2, MON_DX, MON_DY),
MONSTER("PM_DEATH",              "monster-absurd7.png", 3, 2, MON_DX, MON_DY),
MONSTER("PM_DEMOGORGON",         "monster-absurd7.png", 4, 2, MON_DX, MON_DY),
MONSTER("PM_DISPATER",           "monster-absurd7.png", 5, 2, MON_DX, MON_DY),
MONSTER("PM_EARENDIL",           "monster-absurd7.png", 0, 3, MON_DX, MON_DY),
MONSTER("PM_ELWING",             "monster-absurd7.png", 1, 3, MON_DX, MON_DY),
MONSTER("PM_FAMINE",             "monster-absurd7.png", 2, 3, MON_DX, MON_DY),
MONSTER("PM_GERYON",             "monster-absurd7.png", 3, 3, MON_DX, MON_DY),
MONSTER("PM_GOBLIN_KING",        "monster-absurd7.png", 4, 3, MON_DX, MON_DY),
MONSTER("PM_GRAND_MASTER",       "monster-absurd7.png", 5, 3, MON_DX, MON_DY),

MONSTER("PM_HIGH_ELF",           "monster-absurd8.png", 0, 0, MON_DX, MON_DY),
MONSTER("PM_HIPPOCRATES",        "monster-absurd8.png", 1, 0, MON_DX, MON_DY),
MONSTER("PM_IXOTH",              "monster-absurd8.png", 2, 0, MON_DX, MON_DY),
MONSTER("PM_JUIBLEX",            "monster-absurd8.png", 3, 0, MON_DX, MON_DY),
MONSTER("PM_KING_ARTHUR",        "monster-absurd8.png", 4, 0, MON_DX, MON_DY),
MONSTER("PM_LORD_CARNARVON",     "monster-absurd8.png", 5, 0, MON_DX, MON_DY),
MONSTER("PM_LORD_SATO",          "monster-absurd8.png", 0, 1, MON_DX, MON_DY),
MONSTER("PM_LORD_SURTUR",        "monster-absurd8.png", 1, 1, MON_DX, MON_DY),
MONSTER("PM_MASTER_ASSASSIN",    "monster-absurd8.png", 2, 1, MON_DX, MON_DY),
MONSTER("PM_MASTER_KAEN",        "monster-absurd8.png", 3, 1, MON_DX, MON_DY),
MONSTER("PM_MASTER_OF_THIEVES",  "monster-absurd8.png", 4, 1, MON_DX, MON_DY),
MONSTER("PM_MINION_OF_HUHETOTL", "monster-absurd8.png", 5, 1, MON_DX, MON_DY),
MONSTER("PM_NALZOK",             "monster-absurd8.png", 0, 2, MON_DX, MON_DY),
MONSTER("PM_NAZGUL",             "monster-absurd8.png", 1, 2, MON_DX, MON_DY),
MONSTER("PM_NEFERET_THE_GREEN",  "monster-absurd8.png", 2, 2, MON_DX, MON_DY),
MONSTER("PM_NORN",               "monster-absurd8.png", 3, 2, MON_DX, MON_DY),
MONSTER("PM_ORCUS",              "monster-absurd8.png", 4, 2, MON_DX, MON_DY),
MONSTER("PM_ORION",              "monster-absurd8.png", 5, 2, MON_DX, MON_DY),
MONSTER("PM_PELIAS",             "monster-absurd8.png", 0, 3, MON_DX, MON_DY),
MONSTER("PM_PESTILENCE",         "monster-absurd8.png", 1, 3, MON_DX, MON_DY),
MONSTER("PM_SCORPIUS",           "monster-absurd8.png", 2, 3, MON_DX, MON_DY),
MONSTER("PM_SHAMAN_KARNOV",      "monster-absurd8.png", 3, 3, MON_DX, MON_DY),
MONSTER("PM_THOTH_AMON",         "monster-absurd8.png", 4, 3, MON_DX, MON_DY),
MONSTER("PM_TWOFLOWER",          "monster-absurd8.png", 5, 3, MON_DX, MON_DY),

MONSTER("PM_VLAD_THE_IMPALER",   "monster-absurd9.png", 0, 0, MON_DX, MON_DY),
MONSTER("PM_YEENOGHU",           "monster-absurd9.png", 1, 0, MON_DX, MON_DY),
MONSTER("PM_ABBOT",              "monster-absurd9.png", 2, 0, MON_DX, MON_DY),
MONSTER("PM_ACOLYTE",            "monster-absurd9.png", 3, 0, MON_DX, MON_DY),
MONSTER("PM_ALIGNED_PRIEST",     "monster-absurd9.png", 4, 0, MON_DX, MON_DY),
MONSTER("PM_APPRENTICE",         "monster-absurd9.png", 5, 0, MON_DX, MON_DY),
MONSTER("PM_ATTENDANT",          "monster-absurd9.png", 0, 1, MON_DX, MON_DY),
MONSTER("PM_BABY_CROCODILE",     "monster-absurd9.png", 1, 1, MON_DX, MON_DY),
MONSTER("PM_BALROG",             "monster-absurd9.png", 2, 1, MON_DX, MON_DY),
MONSTER("PM_BARBARIAN",          "monster-absurd9.png", 3, 1, MON_DX, MON_DY),
MONSTER("PM_BARBED_DEVIL",       "monster-absurd9.png", 4, 1, MON_DX, MON_DY),
MONSTER("PM_BARROW_WIGHT",       "monster-absurd9.png", 5, 1, MON_DX, MON_DY),
// MONSTER("PM_BLACK_PUDDING",      "monster-absurd9.png", 0, 2, MON_DX, MON_DY),
MONSTER("PM_BONE_DEVIL",         "monster-absurd9.png", 1, 2, MON_DX, MON_DY),
// MONSTER("PM_BROWN_PUDDING",      "monster-absurd9.png", 2, 2, MON_DX, MON_DY),
MONSTER("PM_CAVEMAN",            "monster-absurd9.png", 3, 2, MON_DX, MON_DY),
MONSTER("PM_CAVEWOMAN",          "monster-absurd9.png", 4, 2, MON_DX, MON_DY),
MONSTER("PM_CHAMELEON",          "monster-absurd9.png", 5, 2, MON_DX, MON_DY),
MONSTER("PM_CHIEFTAIN",          "monster-absurd9.png", 0, 3, MON_DX, MON_DY),
MONSTER("PM_CROCODILE",          "monster-absurd9.png", 1, 3, MON_DX, MON_DY),
MONSTER("PM_ELECTRIC_EEL",       "monster-absurd9.png", 2, 3, MON_DX, MON_DY),
MONSTER("PM_ERINYS",             "monster-absurd9.png", 3, 3, MON_DX, MON_DY),
MONSTER("PM_GHOUL",              "monster-absurd9.png", 4, 3, MON_DX, MON_DY),
// MONSTER("PM_GRAY_OOZE",          "monster-absurd9.png", 5, 3, MON_DX, MON_DY),

MONSTER("PM_GUARD",              "monster-absurd10.png", 0, 0, MON_DX, MON_DY),
MONSTER("PM_GUIDE",              "monster-absurd10.png", 1, 0, MON_DX, MON_DY),
MONSTER("PM_HEALER",             "monster-absurd10.png", 2, 0, MON_DX, MON_DY),
MONSTER("PM_HEZROU",             "monster-absurd10.png", 3, 0, MON_DX, MON_DY),
MONSTER("PM_HORNED_DEVIL",       "monster-absurd10.png", 4, 0, MON_DX, MON_DY),
MONSTER("PM_HUNTER",             "monster-absurd10.png", 5, 0, MON_DX, MON_DY),
MONSTER("PM_ICE_DEVIL",          "monster-absurd10.png", 0, 1, MON_DX, MON_DY),
MONSTER("PM_ICE_TROLL",          "monster-absurd10.png", 1, 1, MON_DX, MON_DY),
MONSTER("PM_IGUANA",             "monster-absurd10.png", 2, 1, MON_DX, MON_DY),
MONSTER("PM_MAIL_DAEMON",        "monster-absurd10.png", 3, 1, MON_DX, MON_DY),
MONSTER("PM_MARILITH",           "monster-absurd10.png", 4, 1, MON_DX, MON_DY),
MONSTER("PM_MONK",               "monster-absurd10.png", 5, 1, MON_DX, MON_DY),
MONSTER("PM_NALFESHNEE",         "monster-absurd10.png", 0, 2, MON_DX, MON_DY),
MONSTER("PM_NEANDERTHAL",        "monster-absurd10.png", 1, 2, MON_DX, MON_DY),
MONSTER("PM_NINJA",              "monster-absurd10.png", 2, 2, MON_DX, MON_DY),
MONSTER("PM_PAGE",               "monster-absurd10.png", 3, 2, MON_DX, MON_DY),
MONSTER("PM_PIRANHA",            "monster-absurd10.png", 4, 2, MON_DX, MON_DY),
MONSTER("PM_PIT_FIEND",          "monster-absurd10.png", 5, 2, MON_DX, MON_DY),
MONSTER("PM_PRIESTESS",          "monster-absurd10.png", 0, 3, MON_DX, MON_DY),
MONSTER("PM_PRISONER",           "monster-absurd10.png", 1, 3, MON_DX, MON_DY),
MONSTER("PM_ROSHI",              "monster-absurd10.png", 2, 3, MON_DX, MON_DY),
MONSTER("PM_SALAMANDER",         "monster-absurd10.png", 3, 3, MON_DX, MON_DY),
MONSTER("PM_SANDESTIN",          "monster-absurd10.png", 4, 3, MON_DX, MON_DY),
MONSTER("PM_SHADE",              "monster-absurd10.png", 5, 3, MON_DX, MON_DY),

MONSTER("PM_SHIMMERING_DRAGON",  "monster-absurd11.png", 0, 0, MON_DX, MON_DY),
MONSTER("PM_STUDENT",            "monster-absurd11.png", 1, 0, MON_DX, MON_DY),
MONSTER("PM_THUG",               "monster-absurd11.png", 2, 0, MON_DX, MON_DY),
MONSTER("PM_UMBER_HULK",         "monster-absurd11.png", 3, 0, MON_DX, MON_DY),
MONSTER("PM_VAMPIRE_LORD",       "monster-absurd11.png", 4, 0, MON_DX, MON_DY),
MONSTER("PM_VAMPIRE_MAGE",       "monster-absurd11.png", 5, 0, MON_DX, MON_DY),
MONSTER("PM_VORPAL_JABBERWOCK",  "monster-absurd11.png", 0, 1, MON_DX, MON_DY),
MONSTER("PM_VROCK",              "monster-absurd11.png", 1, 1, MON_DX, MON_DY),
MONSTER("PM_WARRIOR",            "monster-absurd11.png", 2, 1, MON_DX, MON_DY),

/* statues: NAME                   FILENAME          x coord     y coord    width       height*/
/* The name field is the name of a define in pm.h */
MONSTER("PM_KNIGHT",            "monster6.png", 0, 0, MON_DX, MON_DY),
// MONSTER("PM_GOBLIN",            "monster6.png", 1, 0, MON_DX, MON_DY),
MONSTER("PM_QUEEN_BEE",         "monster6.png", 2, 0, MON_DX, MON_DY),
MONSTER("PM_FIRE_ELEMENTAL",    "monster6.png", 3, 0, MON_DX, MON_DY),
// MONSTER("PM_GREEN_SLIME",       "monster6.png", 4, 0, MON_DX, MON_DY),
MONSTER("PM_SKELETON",          "monster6.png", 5, 0, MON_DX, MON_DY),
MONSTER("PM_FLOATING_EYE",      "monster6.png", 0, 1, MON_DX, MON_DY),
MONSTER("PM_QUANTUM_MECHANIC",  "monster6.png", 1, 1, MON_DX, MON_DY),
MONSTER("PM_HOUSECAT",          "monster6.png", 2, 1, MON_DX, MON_DY),
MONSTER("PM_LIZARD",            "monster6.png", 3, 1, MON_DX, MON_DY),
MONSTER("PM_LITTLE_DOG",        "monster6.png", 4, 1, MON_DX, MON_DY),
MONSTER("PM_TROLL",             "monster6.png", 5, 1, MON_DX, MON_DY),
MONSTER("PM_WOOD_NYMPH",        "monster6.png", 0, 2, MON_DX, MON_DY),
MONSTER("PM_MOUNTAIN_NYMPH",    "monster6.png", 1, 2, MON_DX, MON_DY),
MONSTER("PM_WIZARD",            "monster6.png", 2, 2, MON_DX, MON_DY),
MONSTER("PM_VALKYRIE",          "monster6.png", 3, 2, MON_DX, MON_DY),
MONSTER("PM_RANGER",            "monster6.png", 4, 2, MON_DX, MON_DY),
MONSTER("PM_WATER_NYMPH",       "monster6.png", 5, 2, MON_DX, MON_DY),
MONSTER("PM_GHOST",             "monster6.png", 0, 3, MON_DX, MON_DY),
MONSTER("PM_WRAITH",            "monster6.png", 1, 3, MON_DX, MON_DY),
MONSTER("PM_GIANT_BEETLE",      "monster6.png", 2, 3, MON_DX, MON_DY),
MONSTER("PM_HUMAN_ZOMBIE",      "monster6.png", 3, 3, MON_DX, MON_DY),
MONSTER("PM_CENTIPEDE",         "monster6.png", 4, 3, MON_DX, MON_DY),
MONSTER("PM_GIANT_RAT",         "monster6.png", 5, 3, MON_DX, MON_DY),
MONSTER("PM_ARCHEOLOGIST",      "monster7.png", 0, 0, MON_DX, MON_DY),
MONSTER("PM_TOURIST",           "monster7.png", 1, 0, MON_DX, MON_DY),
MONSTER("PM_ROGUE",             "monster7.png", 2, 0, MON_DX, MON_DY),
MONSTER("PM_PRIEST",            "monster7.png", 3, 0, MON_DX, MON_DY),
MONSTER("PM_COYOTE",            "monster7.png", 4, 0, MON_DX, MON_DY),
MONSTER("PM_JACKAL",            "monster7.png", 5, 0, MON_DX, MON_DY),
MONSTER("PM_VAMPIRE_BAT",       "monster7.png", 0, 1, MON_DX, MON_DY),
MONSTER("PM_FOREST_CENTAUR",    "monster7.png", 1, 1, MON_DX, MON_DY),
MONSTER("PM_SOLDIER_ANT",       "monster7.png", 2, 1, MON_DX, MON_DY),
MONSTER("PM_KILLER_BEE",        "monster7.png", 3, 1, MON_DX, MON_DY),
MONSTER("PM_YELLOW_LIGHT",      "monster7.png", 4, 1, MON_DX, MON_DY),
MONSTER("PM_FOX",               "monster7.png", 5, 1, MON_DX, MON_DY),
MONSTER("PM_COCKATRICE",        "monster7.png", 0, 2, MON_DX, MON_DY),
MONSTER("PM_KRAKEN",            "monster7.png", 1, 2, MON_DX, MON_DY),
MONSTER("PM_GIANT_EEL",         "monster7.png", 2, 2, MON_DX, MON_DY),
MONSTER("PM_PYROLISK",          "monster7.png", 3, 2, MON_DX, MON_DY),
// MONSTER("PM_YELLOW_MOLD",       "monster7.png", 4, 2, MON_DX, MON_DY),
MONSTER("PM_SHARK",             "monster7.png", 5, 2, MON_DX, MON_DY),
MONSTER("PM_CHICKATRICE",       "monster7.png", 0, 3, MON_DX, MON_DY),
MONSTER("PM_SMALL_MIMIC",       "monster7.png", 1, 3, MON_DX, MON_DY),
MONSTER("PM_LARGE_MIMIC",       "monster7.png", 2, 3, MON_DX, MON_DY),
MONSTER("PM_GIANT_MIMIC",       "monster7.png", 3, 3, MON_DX, MON_DY),
// MONSTER("PM_RED_MOLD",          "monster7.png", 4, 3, MON_DX, MON_DY),
// MONSTER("PM_VIOLET_FUNGUS",     "monster7.png", 5, 3, MON_DX, MON_DY),
// MONSTER("PM_ACID_BLOB",         "monster8.png", 0, 0, MON_DX, MON_DY),
// MONSTER("PM_QUIVERING_BLOB",    "monster8.png", 1, 0, MON_DX, MON_DY),
// MONSTER("PM_GELATINOUS_CUBE",   "monster8.png", 2, 0, MON_DX, MON_DY),
// MONSTER("PM_BROWN_MOLD",        "monster8.png", 3, 0, MON_DX, MON_DY),
// MONSTER("PM_LICHEN",            "monster8.png", 4, 0, MON_DX, MON_DY),
// MONSTER("PM_GREEN_MOLD",        "monster8.png", 5, 0, MON_DX, MON_DY),
// MONSTER("PM_SHRIEKER",          "monster8.png", 0, 1, MON_DX, MON_DY),
MONSTER("PM_STALKER",           "monster8.png", 1, 1, MON_DX, MON_DY),
MONSTER("PM_AIR_ELEMENTAL",     "monster8.png", 2, 1, MON_DX, MON_DY),
MONSTER("PM_EARTH_ELEMENTAL",   "monster8.png", 3, 1, MON_DX, MON_DY),
MONSTER("PM_WATER_ELEMENTAL",   "monster8.png", 4, 1, MON_DX, MON_DY),
MONSTER("PM_BLACK_LIGHT",       "monster8.png", 5, 1, MON_DX, MON_DY),
MONSTER("PM_ROCK_PIERCER",      "monster8.png", 0, 2, MON_DX, MON_DY),
MONSTER("PM_IRON_PIERCER",      "monster8.png", 1, 2, MON_DX, MON_DY),
MONSTER("PM_GLASS_PIERCER",     "monster8.png", 2, 2, MON_DX, MON_DY),
MONSTER("PM_DOG",               "monster8.png", 3, 2, MON_DX, MON_DY),
MONSTER("PM_LARGE_DOG",         "monster8.png", 4, 2, MON_DX, MON_DY),
MONSTER("PM_WATER_DEMON",       "monster8.png", 5, 2, MON_DX, MON_DY),
MONSTER("PM_GECKO",             "monster8.png", 0, 3, MON_DX, MON_DY),
MONSTER("PM_NEWT",              "monster8.png", 1, 3, MON_DX, MON_DY),
MONSTER("PM_SHOPKEEPER",        "monster8.png", 2, 3, MON_DX, MON_DY),
MONSTER("PM_GAS_SPORE",         "monster8.png", 3, 3, MON_DX, MON_DY),
MONSTER("PM_KITTEN",            "monster8.png", 4, 3, MON_DX, MON_DY),
MONSTER("PM_LARGE_CAT",         "monster8.png", 5, 3, MON_DX, MON_DY),
MONSTER("PM_FREEZING_SPHERE",   "monster9.png", 0, 0, MON_DX, MON_DY),
MONSTER("PM_FLAMING_SPHERE",    "monster9.png", 1, 0, MON_DX, MON_DY),
MONSTER("PM_SHOCKING_SPHERE",   "monster9.png", 2, 0, MON_DX, MON_DY),
MONSTER("PM_PONY",              "monster9.png", 3, 0, MON_DX, MON_DY),
MONSTER("PM_HORSE",             "monster9.png", 4, 0, MON_DX, MON_DY),
MONSTER("PM_WARHORSE",          "monster9.png", 5, 0, MON_DX, MON_DY),
MONSTER("PM_ROTHE",             "monster9.png", 0, 1, MON_DX, MON_DY),
MONSTER("PM_GNOME",             "monster9.png", 1, 1, MON_DX, MON_DY),
MONSTER("PM_GREEN_ELF",         "monster9.png", 2, 1, MON_DX, MON_DY),
MONSTER("PM_DWARF",             "monster9.png", 3, 1, MON_DX, MON_DY),
MONSTER("PM_WATER_MOCCASIN",    "monster9.png", 4, 1, MON_DX, MON_DY),
// MONSTER("PM_MANES",             "monster9.png", 5, 1, MON_DX, MON_DY),
MONSTER("PM_HOBBIT",            "monster9.png", 0, 2, MON_DX, MON_DY),
MONSTER("PM_BLACK_UNICORN",     "monster9.png", 1, 2, MON_DX, MON_DY),
MONSTER("PM_GRAY_UNICORN",      "monster9.png", 2, 2, MON_DX, MON_DY),
MONSTER("PM_WHITE_UNICORN",     "monster9.png", 3, 2, MON_DX, MON_DY),
MONSTER("PM_GIANT_ANT",         "monster9.png", 4, 2, MON_DX, MON_DY),
MONSTER("PM_FIRE_ANT",          "monster9.png", 5, 2, MON_DX, MON_DY),
MONSTER("PM_ELVENKING",         "monster9.png", 0, 3, MON_DX, MON_DY),
MONSTER("PM_ELF_LORD",          "monster9.png", 1, 3, MON_DX, MON_DY),
MONSTER("PM_GREY_ELF",          "monster9.png", 2, 3, MON_DX, MON_DY),
MONSTER("PM_WOODLAND_ELF",      "monster9.png", 3, 3, MON_DX, MON_DY),
MONSTER("PM_ELF",               "monster9.png", 4, 3, MON_DX, MON_DY),
MONSTER("PM_ROCK_MOLE",         "monster9.png", 5, 3, MON_DX, MON_DY),
MONSTER("PM_DWARF_LORD",        "monster0.png", 0, 0, MON_DX, MON_DY),
MONSTER("PM_DWARF_KING",        "monster0.png", 1, 0, MON_DX, MON_DY),
MONSTER("PM_GNOME_LORD",        "monster0.png", 2, 0, MON_DX, MON_DY),
MONSTER("PM_GNOME_KING",        "monster0.png", 3, 0, MON_DX, MON_DY),
MONSTER("PM_GNOMISH_WIZARD",    "monster0.png", 4, 0, MON_DX, MON_DY),
// MONSTER("PM_ORC",               "monster0.png", 5, 0, MON_DX, MON_DY),
// MONSTER("PM_HILL_ORC",          "monster0.png", 0, 1, MON_DX, MON_DY),
// MONSTER("PM_MORDOR_ORC",        "monster0.png", 1, 1, MON_DX, MON_DY),
// MONSTER("PM_URUK_HAI",          "monster0.png", 2, 1, MON_DX, MON_DY),
// MONSTER("PM_HOBGOBLIN",         "monster0.png", 3, 1, MON_DX, MON_DY),
MONSTER("PM_BUGBEAR",           "monster0.png", 4, 1, MON_DX, MON_DY),
// MONSTER("PM_KOBOLD",            "monster0.png", 5, 1, MON_DX, MON_DY),
// MONSTER("PM_LARGE_KOBOLD",      "monster0.png", 0, 2, MON_DX, MON_DY),
// MONSTER("PM_KOBOLD_SHAMAN",     "monster0.png", 1, 2, MON_DX, MON_DY),
// MONSTER("PM_KOBOLD_LORD",       "monster0.png", 2, 2, MON_DX, MON_DY),
// MONSTER("PM_ORC_CAPTAIN",       "monster0.png", 3, 2, MON_DX, MON_DY),
// MONSTER("PM_ORC_SHAMAN",        "monster0.png", 4, 2, MON_DX, MON_DY),
MONSTER("PM_LEPRECHAUN",        "monster0.png", 5, 2, MON_DX, MON_DY),
MONSTER("PM_GARTER_SNAKE",      "monster0.png", 0, 3, MON_DX, MON_DY),
MONSTER("PM_SNAKE",             "monster0.png", 1, 3, MON_DX, MON_DY),
MONSTER("PM_PIT_VIPER",         "monster0.png", 2, 3, MON_DX, MON_DY),
MONSTER("PM_PYTHON",            "monster0.png", 3, 3, MON_DX, MON_DY),
MONSTER("PM_COBRA",             "monster0.png", 4, 3, MON_DX, MON_DY),
MONSTER("PM_WUMPUS",            "monster0.png", 5, 3, MON_DX, MON_DY),
MONSTER("PM_KOBOLD_ZOMBIE",     "monster1.png", 0, 0, MON_DX, MON_DY),
MONSTER("PM_GNOME_ZOMBIE",      "monster1.png", 1, 0, MON_DX, MON_DY),
MONSTER("PM_DWARF_ZOMBIE",      "monster1.png", 2, 0, MON_DX, MON_DY),
MONSTER("PM_ORC_ZOMBIE",        "monster1.png", 3, 0, MON_DX, MON_DY),
MONSTER("PM_ELF_ZOMBIE",        "monster1.png", 4, 0, MON_DX, MON_DY),
MONSTER("PM_KOBOLD_MUMMY",      "monster1.png", 5, 0, MON_DX, MON_DY),
MONSTER("PM_GNOME_MUMMY",       "monster1.png", 0, 1, MON_DX, MON_DY),
MONSTER("PM_DWARF_MUMMY",       "monster1.png", 1, 1, MON_DX, MON_DY),
MONSTER("PM_ORC_MUMMY",         "monster1.png", 2, 1, MON_DX, MON_DY),
MONSTER("PM_HUMAN_MUMMY",       "monster1.png", 3, 1, MON_DX, MON_DY),
MONSTER("PM_ELF_MUMMY",         "monster1.png", 4, 1, MON_DX, MON_DY),
MONSTER("PM_HOMUNCULUS",        "monster1.png", 5, 1, MON_DX, MON_DY),
MONSTER("PM_IMP",               "monster1.png", 0, 2, MON_DX, MON_DY),
// MONSTER("PM_LEMURE",            "monster1.png", 1, 2, MON_DX, MON_DY),
MONSTER("PM_QUASIT",            "monster1.png", 2, 2, MON_DX, MON_DY),
MONSTER("PM_TENGU",             "monster1.png", 3, 2, MON_DX, MON_DY),
MONSTER("PM_ORACLE",            "monster1.png", 4, 2, MON_DX, MON_DY),
MONSTER("PM_WATCHMAN",          "monster1.png", 3, 3, MON_DX, MON_DY),
MONSTER("PM_WATCH_CAPTAIN",     "monster1.png", 4, 3, MON_DX, MON_DY),
MONSTER("PM_SOLDIER",           "monster1.png", 5, 3, MON_DX, MON_DY),
MONSTER("PM_SERGEANT",          "monster2.png", 0, 0, MON_DX, MON_DY),
MONSTER("PM_LIEUTENANT",        "monster2.png", 1, 0, MON_DX, MON_DY),
MONSTER("PM_CAPTAIN",           "monster2.png", 2, 0, MON_DX, MON_DY),
MONSTER("PM_GRID_BUG",          "monster2.png", 3, 0, MON_DX, MON_DY),
MONSTER("PM_XAN",               "monster2.png", 4, 0, MON_DX, MON_DY),
MONSTER("PM_GARGOYLE",          "monster2.png", 5, 0, MON_DX, MON_DY),
MONSTER("PM_WINGED_GARGOYLE",   "monster2.png", 0, 1, MON_DX, MON_DY),
MONSTER("PM_DINGO",             "monster2.png", 1, 1, MON_DX, MON_DY),
MONSTER("PM_WOLF",              "monster2.png", 2, 1, MON_DX, MON_DY),
MONSTER("PM_WEREWOLF",          "monster2.png", 3, 1, MON_DX, MON_DY),
MONSTER("PM_WARG",              "monster2.png", 4, 1, MON_DX, MON_DY),
MONSTER("PM_WINTER_WOLF_CUB",   "monster2.png", 5, 1, MON_DX, MON_DY),
MONSTER("PM_WINTER_WOLF",       "monster2.png", 0, 2, MON_DX, MON_DY),
MONSTER("PM_HELL_HOUND_PUP",    "monster2.png", 1, 2, MON_DX, MON_DY),
MONSTER("PM_HELL_HOUND",        "monster2.png", 2, 2, MON_DX, MON_DY),
MONSTER("PM_JAGUAR",            "monster2.png", 3, 2, MON_DX, MON_DY),
MONSTER("PM_LYNX",              "monster2.png", 4, 2, MON_DX, MON_DY),
MONSTER("PM_PANTHER",           "monster2.png", 5, 2, MON_DX, MON_DY),
MONSTER("PM_TIGER",             "monster2.png", 0, 3, MON_DX, MON_DY),
MONSTER("PM_MIND_FLAYER",       "monster2.png", 1, 3, MON_DX, MON_DY),
MONSTER("PM_MASTER_MIND_FLAYER","monster2.png", 2, 3, MON_DX, MON_DY),
MONSTER("PM_LICH",              "monster2.png", 3, 3, MON_DX, MON_DY),
MONSTER("PM_DEMILICH",          "monster2.png", 4, 3, MON_DX, MON_DY),
MONSTER("PM_MASTER_LICH",       "monster2.png", 5, 3, MON_DX, MON_DY),
MONSTER("PM_ARCH_LICH",         "monster3.png", 0, 0, MON_DX, MON_DY),
MONSTER("PM_GUARDIAN_NAGA",     "monster3.png", 1, 0, MON_DX, MON_DY),
MONSTER("PM_GUARDIAN_NAGA_HATCHLING","monster3.png", 2, 0, MON_DX, MON_DY),
MONSTER("PM_GOLDEN_NAGA",       "monster3.png", 3, 0, MON_DX, MON_DY),
MONSTER("PM_GOLDEN_NAGA_HATCHLING","monster3.png", 4, 0, MON_DX, MON_DY),
MONSTER("PM_RED_NAGA",          "monster3.png", 5, 0, MON_DX, MON_DY),
MONSTER("PM_RED_NAGA_HATCHLING","monster3.png", 0, 1, MON_DX, MON_DY),
MONSTER("PM_BLACK_NAGA",        "monster3.png", 1, 1, MON_DX, MON_DY),
MONSTER("PM_BLACK_NAGA_HATCHLING","monster3.png", 2, 1, MON_DX, MON_DY),
MONSTER("PM_RUST_MONSTER",      "monster3.png", 3, 1, MON_DX, MON_DY),
MONSTER("PM_GREMLIN",           "monster3.png", 4, 1, MON_DX, MON_DY),
MONSTER("PM_BABY_PURPLE_WORM",  "monster3.png", 5, 1, MON_DX, MON_DY),
MONSTER("PM_MONKEY",            "monster3.png", 0, 2, MON_DX, MON_DY),
MONSTER("PM_APE",               "monster3.png", 1, 2, MON_DX, MON_DY),
MONSTER("PM_OWLBEAR",           "monster3.png", 2, 2, MON_DX, MON_DY),
MONSTER("PM_YETI",              "monster3.png", 3, 2, MON_DX, MON_DY),
MONSTER("PM_CARNIVOROUS_APE",   "monster3.png", 4, 2, MON_DX, MON_DY),
MONSTER("PM_STRAW_GOLEM",       "monster3.png", 5, 2, MON_DX, MON_DY),
MONSTER("PM_PAPER_GOLEM",       "monster3.png", 0, 3, MON_DX, MON_DY),
MONSTER("PM_ROPE_GOLEM",        "monster3.png", 1, 3, MON_DX, MON_DY),
MONSTER("PM_GOLD_GOLEM",        "monster3.png", 2, 3, MON_DX, MON_DY),
MONSTER("PM_LEATHER_GOLEM",     "monster3.png", 3, 3, MON_DX, MON_DY),
MONSTER("PM_WOOD_GOLEM",        "monster3.png", 4, 3, MON_DX, MON_DY),
MONSTER("PM_FLESH_GOLEM",       "monster3.png", 5, 3, MON_DX, MON_DY),
MONSTER("PM_SCORPION",          "monster10.png", 0, 0, MON_DX, MON_DY),
MONSTER("PM_CAVE_SPIDER",       "monster10.png", 1, 0, MON_DX, MON_DY),
MONSTER("PM_GIANT_SPIDER",      "monster10.png", 2, 0, MON_DX, MON_DY),
MONSTER("PM_BAT",               "monster10.png", 3, 0, MON_DX, MON_DY),
MONSTER("PM_GIANT_BAT",         "monster10.png", 4, 0, MON_DX, MON_DY),
MONSTER("PM_RAVEN",             "monster10.png", 5, 0, MON_DX, MON_DY),
MONSTER("PM_DJINNI",            "monster10.png", 0, 1, MON_DX, MON_DY),
MONSTER("PM_MOUNTAIN_CENTAUR",  "monster10.png", 1, 1, MON_DX, MON_DY),
MONSTER("PM_PLAINS_CENTAUR",    "monster10.png", 2, 1, MON_DX, MON_DY),
MONSTER("PM_COUATL",            "monster10.png", 3, 1, MON_DX, MON_DY),
MONSTER("PM_DOPPELGANGER",      "monster10.png", 4, 1, MON_DX, MON_DY),
MONSTER("PM_CROESUS",           "monster10.png", 5, 1, MON_DX, MON_DY),
MONSTER("PM_KEYSTONE_KOP",      "monster10.png", 0, 2, MON_DX, MON_DY),
MONSTER("PM_KOP_SERGEANT",      "monster10.png", 1, 2, MON_DX, MON_DY),
MONSTER("PM_KOP_LIEUTENANT",    "monster10.png", 2, 2, MON_DX, MON_DY),
MONSTER("PM_KOP_KAPTAIN",       "monster10.png", 3, 2, MON_DX, MON_DY),
MONSTER("PM_VAMPIRE",           "monster10.png", 4, 2, MON_DX, MON_DY),
MONSTER("PM_SEWER_RAT",         "monster10.png", 5, 2, MON_DX, MON_DY),
MONSTER("PM_RABID_RAT",         "monster10.png", 0, 3, MON_DX, MON_DY),
MONSTER("PM_WERERAT",           "monster10.png", 1, 3, MON_DX, MON_DY),
MONSTER("PM_WOODCHUCK",         "monster10.png", 2, 3, MON_DX, MON_DY),
MONSTER("PM_MEDUSA",            "monster10.png", 3, 3, MON_DX, MON_DY),
MONSTER("PM_HIGH_PRIEST",       "monster10.png", 4, 3, MON_DX, MON_DY),
MONSTER("PM_NURSE",             "monster10.png", 5, 3, MON_DX, MON_DY),
MONSTER("PM_SAMURAI",           "monster11.png", 0, 0, MON_DX, MON_DY),
/* slashem only */
MONSTER("PM_NECROMANCER",       "monster11.png", 1, 0, MON_DX, MON_DY),
MONSTER("PM_ICE_MAGE",          "monster11.png", 2, 0, MON_DX, MON_DY),
MONSTER("PM_FLAME_MAGE",        "monster11.png", 3, 0, MON_DX, MON_DY),
/* both */
MONSTER("PM_JELLYFISH",         "monster11.png", 4, 0, MON_DX, MON_DY),
MONSTER("PM_GIANT",             "monster4.png", 0, 0, MON_DX, MON_LARGE_DY),
MONSTER("PM_STONE_GIANT",       "monster4.png", 1, 0, MON_DX, MON_LARGE_DY),
MONSTER("PM_HILL_GIANT",        "monster4.png", 2, 0, MON_DX, MON_LARGE_DY),
MONSTER("PM_FROST_GIANT",       "monster4.png", 3, 0, MON_DX, MON_LARGE_DY),
MONSTER("PM_FIRE_GIANT",        "monster4.png", 4, 0, MON_DX, MON_LARGE_DY),
MONSTER("PM_STORM_GIANT",       "monster4.png", 5, 0, MON_DX, MON_LARGE_DY),
MONSTER("PM_ETTIN",             "monster4.png", 0, 1, MON_DX, MON_LARGE_DY),
MONSTER("PM_TITAN",             "monster4.png", 1, 1, MON_DX, MON_LARGE_DY),
MONSTER("PM_MINOTAUR",          "monster4.png", 2, 1, MON_DX, MON_LARGE_DY),
MONSTER("PM_GIANT_ZOMBIE",      "monster4.png", 3, 1, MON_DX, MON_LARGE_DY),
MONSTER("PM_ETTIN_ZOMBIE",      "monster4.png", 4, 1, MON_DX, MON_LARGE_DY),
MONSTER("PM_GIANT_MUMMY",       "monster4.png", 5, 1, MON_DX, MON_LARGE_DY),
MONSTER("PM_ETTIN_MUMMY",       "monster4.png", 0, 2, MON_DX, MON_LARGE_DY),
MONSTER("PM_OGRE",              "monster4.png", 1, 2, MON_DX, MON_LARGE_DY),
MONSTER("PM_OGRE_LORD",         "monster4.png", 2, 2, MON_DX, MON_LARGE_DY),
MONSTER("PM_OGRE_KING",         "monster4.png", 3, 2, MON_DX, MON_LARGE_DY),
MONSTER("PM_DISENCHANTER",      "monster4.png", 4, 2, MON_DX, MON_LARGE_DY),
MONSTER("PM_SASQUATCH",         "monster4.png", 5, 2, MON_DX, MON_LARGE_DY),
MONSTER("PM_SUCCUBUS",          "monster5.png", 0, 0, MON_LARGE_DX, MON_LARGE_DY),
MONSTER("PM_INCUBUS",           "monster5.png", 1, 0, MON_LARGE_DX, MON_LARGE_DY),
MONSTER("PM_BABY_YELLOW_DRAGON","monster5.png", 2, 0, MON_LARGE_DX, MON_LARGE_DY),
MONSTER("PM_BABY_WHITE_DRAGON", "monster5.png", 3, 0, MON_LARGE_DX, MON_LARGE_DY),
MONSTER("PM_BABY_SILVER_DRAGON","monster5.png", 4, 0, MON_LARGE_DX, MON_LARGE_DY),
MONSTER("PM_BABY_RED_DRAGON",   "monster5.png", 0, 1, MON_LARGE_DX, MON_LARGE_DY),
MONSTER("PM_BABY_BLACK_DRAGON", "monster5.png", 1, 1, MON_LARGE_DX, MON_LARGE_DY),
MONSTER("PM_BABY_GRAY_DRAGON",  "monster5.png", 2, 1, MON_LARGE_DX, MON_LARGE_DY),
MONSTER("PM_BABY_GREEN_DRAGON", "monster5.png", 3, 1, MON_LARGE_DX, MON_LARGE_DY),
MONSTER("PM_BABY_ORANGE_DRAGON","monster5.png", 4, 1, MON_LARGE_DX, MON_LARGE_DY),
MONSTER("PM_BABY_BLUE_DRAGON",  "monster5.png", 0, 2, MON_LARGE_DX, MON_LARGE_DY),
MONSTER("PM_PURPLE_WORM",       "monster5.png", 1, 2, MON_LARGE_DX, MON_LARGE_DY),
MONSTER("PM_LONG_WORM_TAIL",    "monster5.png", 2, 2, MON_LARGE_DX, MON_LARGE_DY),
MONSTER("PM_LONG_WORM",         "monster5.png", 3, 2, MON_LARGE_DX, MON_LARGE_DY),
MONSTER("PM_BABY_LONG_WORM",    "monster5.png", 4, 2, MON_LARGE_DX, MON_LARGE_DY),
MONSTER("PM_MUMAK",             "monster16.png", 0, 0, MON_DX, MON_LARGE_DY),
MONSTER("PM_CLAY_GOLEM",        "monster16.png", 1, 0, MON_DX, MON_LARGE_DY),
MONSTER("PM_STONE_GOLEM",       "monster16.png", 2, 0, MON_DX, MON_LARGE_DY),
MONSTER("PM_GLASS_GOLEM",       "monster16.png", 3, 0, MON_DX, MON_LARGE_DY),
MONSTER("PM_IRON_GOLEM",        "monster16.png", 4, 0, MON_DX, MON_LARGE_DY),
MONSTER("PM_LURKER_ABOVE",      "monster16.png", 5, 0, MON_DX, MON_LARGE_DY),
MONSTER("PM_TRAPPER",           "monster16.png", 0, 1, MON_DX, MON_LARGE_DY),
MONSTER("PM_ROCK_TROLL",        "monster16.png", 1, 1, MON_DX, MON_LARGE_DY),
MONSTER("PM_WATER_TROLL",       "monster16.png", 2, 1, MON_DX, MON_LARGE_DY),
MONSTER("PM_OLOG_HAI",          "monster16.png", 3, 1, MON_DX, MON_LARGE_DY),
MONSTER("PM_ZRUTY",             "monster18.png", 0, 0, MON_LARGE_DX, MON_LARGE_DY),
MONSTER("PM_LEOCROTTA",         "monster18.png", 1, 0, MON_LARGE_DX, MON_LARGE_DY),
MONSTER("PM_TITANOTHERE",       "monster18.png", 2, 0, MON_LARGE_DX, MON_LARGE_DY),
MONSTER("PM_XORN",              "monster18.png", 3, 0, MON_LARGE_DX, MON_LARGE_DY),
MONSTER("PM_JABBERWOCK",        "monster18.png", 4, 0, MON_LARGE_DX, MON_LARGE_DY),
MONSTER("PM_ANGEL",             "monster13.png", 0, 0, MON_HUGE_DX, MON_HUGE_DY),
MONSTER("PM_KI_RIN",            "monster13.png", 1, 0, MON_HUGE_DX, MON_HUGE_DY),
MONSTER("PM_ARCHON",            "monster13.png", 2, 0, MON_HUGE_DX, MON_HUGE_DY),
MONSTER("PM_SILVER_DRAGON",     "monster14.png", 0, 0, MON_HUGE_DX, MON_HUGE_DY),
MONSTER("PM_BLACK_DRAGON",      "monster14.png", 1, 0, MON_HUGE_DX, MON_HUGE_DY),
MONSTER("PM_WHITE_DRAGON",      "monster14.png", 2, 0, MON_HUGE_DX, MON_HUGE_DY),
MONSTER("PM_BALUCHITHERIUM",    "monster14.png", 0, 1, MON_HUGE_DX, MON_HUGE_DY),
MONSTER("PM_MASTODON",          "monster14.png", 1, 1, MON_HUGE_DX, MON_HUGE_DY),
MONSTER("PM_WIZARD_OF_YENDOR",  "monster14.png", 2, 1, MON_HUGE_DX, MON_HUGE_DY),
MONSTER("PM_RED_DRAGON",        "monster15.png", 0, 0, MON_HUGE_DX, MON_HUGE_DY),
MONSTER("PM_GREEN_DRAGON",      "monster15.png", 1, 0, MON_HUGE_DX, MON_HUGE_DY),
MONSTER("PM_ORANGE_DRAGON",     "monster15.png", 2, 0, MON_HUGE_DX, MON_HUGE_DY),
MONSTER("PM_YELLOW_DRAGON",     "monster15.png", 0, 1, MON_HUGE_DX, MON_HUGE_DY),
MONSTER("PM_BLUE_DRAGON",       "monster15.png", 1, 1, MON_HUGE_DX, MON_HUGE_DY),
MONSTER("PM_GRAY_DRAGON",       "monster15.png", 2, 1, MON_HUGE_DX, MON_HUGE_DY),

/* Jellies, fungi and blobs */
/* statues: NAME                    FILENAME           x coord     y coord    width       height*/
MONSTER("PM_OCHRE_JELLY",        "jellies_fungi.png", 0, 0, MON_DX, MON_DY),
MONSTER("PM_BLUE_JELLY",         "jellies_fungi.png", 1, 0, MON_DX, MON_DY),
MONSTER("PM_GRAY_OOZE",          "jellies_fungi.png", 2, 0, MON_DX, MON_DY),
MONSTER("PM_BLACK_PUDDING",      "jellies_fungi.png", 3, 0, MON_DX, MON_DY),
MONSTER("PM_GREEN_SLIME",        "jellies_fungi.png", 4, 0, MON_DX, MON_DY),
MONSTER("PM_SPOTTED_JELLY",      "jellies_fungi.png", 5, 0, MON_DX, MON_DY),
MONSTER("PM_BROWN_PUDDING",      "jellies_fungi.png", 0, 1, MON_DX, MON_DY),
MONSTER("PM_ORANGE_JELLY",       "jellies_fungi.png", 1, 1, MON_DX, MON_DY),
MONSTER("PM_YELLOW_JELLY",       "jellies_fungi.png", 2, 1, MON_DX, MON_DY),
MONSTER("PM_RANCID_JELLY",       "jellies_fungi.png", 3, 1, MON_DX, MON_DY),
MONSTER("PM_CLEAR_JELLY",        "jellies_fungi.png", 4, 1, MON_DX, MON_DY),
MONSTER("PM_LICHEN",             "jellies_fungi.png", 0, 2, MON_DX, MON_DY),
MONSTER("PM_GREEN_MOLD",         "jellies_fungi.png", 1, 2, MON_DX, MON_DY),
MONSTER("PM_YELLOW_MOLD",        "jellies_fungi.png", 2, 2, MON_DX, MON_DY),
MONSTER("PM_BROWN_MOLD",         "jellies_fungi.png", 3, 2, MON_DX, MON_DY),
MONSTER("PM_SHRIEKER",           "jellies_fungi.png", 4, 2, MON_DX, MON_DY),
MONSTER("PM_RED_MOLD",           "jellies_fungi.png", 5, 2, MON_DX, MON_DY),
MONSTER("PM_VIOLET_FUNGUS",      "jellies_fungi.png", 0, 3, MON_DX, MON_DY),
MONSTER("PM_BLACK_MOLD",         "jellies_fungi.png", 1, 3, MON_DX, MON_DY),
MONSTER("PM_DISGUSTING_MOLD",    "jellies_fungi.png", 2, 3, MON_DX, MON_DY),
MONSTER("PM_ACID_BLOB",          "blobs.png", 0, 0, MON_DX, MON_DY),
MONSTER("PM_QUIVERING_BLOB",     "blobs.png", 1, 0, MON_DX, MON_DY),
MONSTER("PM_GELATINOUS_CUBE",    "blobs.png", 2, 0, MON_DX, MON_DY),
MONSTER("PM_JIGGLING_BLOB",      "blobs.png", 3, 0, MON_DX, MON_DY),
MONSTER("PM_LAVA_BLOB",          "blobs.png", 4, 0, MON_DX, MON_DY),
MONSTER("PM_STATIC_BLOB",        "blobs.png", 5, 0, MON_DX, MON_DY),
MONSTER("PM_BURBLING_BLOB",      "blobs.png", 0, 1, MON_DX, MON_DY),

/* Were foo when in human form */
/* statues: NAME                    FILENAME        x coord     y coord    width       height*/
MONSTER("PM_HUMAN_WERERAT",      "monster-add.png", 0, 0, MON_DX, MON_DY),
MONSTER("PM_HUMAN_WEREJACKAL",   "monster-add.png", 0, 0, MON_DX, MON_DY),
MONSTER("PM_HUMAN_WEREWOLF",     "monster-add.png", 0, 0, MON_DX, MON_DY),

/* cursors: they're actually normal tiles, so they're in here too */
/* statues: NAME                   FILENAME         top left x coord     top left y coord    width       height*/
CURSOR("NORMAL", 1, 1, 16, 23),
CURSOR("SCROLLRIGHT", 21, 1, 19, 22),
CURSOR("SCROLLLEFT", 44, 1, 19, 22),
CURSOR("SCROLLUP", 67, 1, 22, 19),
CURSOR("SCROLLDOWN", 93, 1, 22, 19),
CURSOR("SCROLLUPLEFT", 55, 30, 19, 19),
CURSOR("SCROLLUPRIGHT", 78, 30, 19, 19),
CURSOR("SCROLLDOWNLEFT", 101, 30, 19, 19),
CURSOR("SCROLLDOWNRIGHT", 124, 30, 19, 19),
CURSOR("TARGET_GREEN", 119, 1, 50, 25),
CURSOR("TARGET_RED", 173, 1, 50, 25),
CURSOR("TARGET_INVALID", 227, 1, 46, 25),
CURSOR("TARGET_HELP", 1, 30, 50, 49),
CURSOR("HOURGLASS", 277, 1, 29, 32),
CURSOR("OPENDOOR", 310, 1, 32, 29),
CURSOR("STAIRS", 346, 1, 37, 34),
CURSOR("GOBLET", 312, 34, 24, 34)
};

const int num_tiledefs = sizeof(tiledefinitions)/sizeof(struct tiledef);
const int num_objclassmaps = sizeof(objclassmap)/sizeof(struct classdefmap);
const int num_monclassmaps = sizeof(monclassmap)/sizeof(struct classdefmap);
const int num_eqmapentries = sizeof(tilemap)/sizeof(struct eqmap);
const int num_floor_edges = sizeof(floor_edges)/sizeof(char*);
const int num_fstyles = sizeof(fstyles)/sizeof(struct floorstyles);
const int num_wall_styles = sizeof(wall_styles)/sizeof(char*);

